# -*- encoding: utf-8 -*-


def inspect_(expr):
    r'''Inspects `expr`.

    Factory function.

    Returns inspect agent.
    '''
    from abjad.tools import agenttools
    return agenttools.InspectionAgent(expr)


# -*- encoding: utf-8 -*-
import numbers


def is_integer_equivalent_expr(expr):
    '''Is true when `expr` is an integer-equivalent number.

    ::

        >>> mathtools.is_integer_equivalent_expr(12.0)
        True

    Is true when `expr` evaluates to an integer:

    ::

        >>> mathtools.is_integer_equivalent_expr('12')
        True

    Otherwise false:

    ::

        >>> mathtools.is_integer_equivalent_expr('foo')
        False

    Returns boolean.
    '''
    from abjad.tools import mathtools

    if isinstance(expr, numbers.Number):
        return mathtools.is_integer_equivalent_number(expr)

    try:
        int(expr)
        return True
    except (TypeError, ValueError):
        return False


# -*- encoding: utf-8 -*-
import collections
import types


def increase_elements(sequence, addenda, indices=None):
    '''Increases `sequence` cyclically by `addenda`.

    ..  container:: example

        Increases elements cyclically by ``10`` and ``-10`` in alternation:

        ::

            >>> sequencetools.increase_elements(range(10), [10, -10])
            [10, -9, 12, -7, 14, -5, 16, -3, 18, -1]

    ..  container:: example

        Increases elements by ``0.5`` at indices 0, 4 and 8 and at one element
        following each:

        ::

            >>> sequence = [1, 1, 2, 3, 5, 5, 1, 2, 5, 5, 6]
            >>> sequencetools.increase_elements(
            ...     sequence, [0.5, 0.5], indices=[0, 4, 8])
            [1.5, 1.5, 2, 3, 5.5, 5.5, 1, 2, 5.5, 5.5, 6]

    Returns list.
    '''
    from abjad.tools import sequencetools

    prototype = (collections.Sequence, types.GeneratorType)
    if not isinstance(sequence, prototype):
        raise TypeError(sequence)

    if indices is None:
        result = []
        for i, element in enumerate(sequence):
            new = element + addenda[i % len(addenda)]
            result.append(new)
    else:
        # assert no overlaps
        tmp = [tuple(range(i, len(addenda))) for i in indices]
        tmp = sequencetools.flatten_sequence(tmp)
        assert len(tmp) == len(set(tmp))
        result = sequence[:]
        for i in indices:
            for j in range(len(addenda)):
                result[i + j] += addenda[j]

    return result



# -*- encoding: utf-8 -*-
import copy
import math
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import new


class TaleaRhythmMaker(RhythmMaker):
    r'''Talea rhythm-maker.

    ..  container:: example

        ::

            >>> talea = rhythmmakertools.Talea(
            ...     counts=(1, 2, 3, 4),
            ...     denominator=16,
            ...     )
            >>> maker = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=talea,
            ...     )

        ::

            >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 3/8
                    c'16 [
                    c'8
                    c'8. ]
                }
                {
                    \time 4/8
                    c'4
                    c'16 [
                    c'8
                    c'16 ~ ]
                }
                {
                    \time 3/8
                    c'8
                    c'4
                }
                {
                    \time 4/8
                    c'16 [
                    c'8
                    c'8.
                    c'8 ]
                }
            }

    Follows the two-step configure-once / call-repeatedly pattern shown here.

    Object model of a partially evaluated function that accepts a (possibly
    empty) list of divisions as input and returns a list of selections as
    output (structured one selection per input division).
    '''

    r'''Example helpers:

    # used in a piece with four voices:
    # voice 1 starts reading talea at beginning of talea;
    # voice 2 starts reading talea at second event of talea;
    # voice 3 starts reading talea at third event of talea;
    # voice 4 starts reading talea at fourth event of talea.
    def helper(talea, seeds):
        assert len(seeds) == 2
        if not talea:
            return talea
        voice_index, measure_index = seeds
        talea = sequencetools.rotate_sequence(talea, -voice_index)
        return talea

    # used in a piece with four voices:
    # voice 1 starts reading talea at beginning of talea;
    # voice 2 starts reading talea 1/4 of way through talea;
    # voice 3 starts reading talea 2/4 of way through talea;
    # voice 4 starts reading talea 3/4 of way through talea.
    def quarter_rotation_helper(talea, seeds):
        assert len(seeds) == 2
        if not talea:
            return talea
        voice_index, measure_index = seeds
        index_of_rotation = -voice_index * (len(talea) // 4)
        index_of_rotation += -4 * measure_index
        talea = sequencetools.rotate_sequence(talea, index_of_rotation)
        return talea
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_burnish_specifier',
        '_extra_counts_per_division',
        '_helper_functions',
        '_split_divisions_by_counts',
        '_talea',
        '_talea_denominator',
        )

    _class_name_abbreviation = 'TlRM'

    _human_readable_class_name = 'talea rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        talea=None,
        split_divisions_by_counts=None,
        extra_counts_per_division=None,
        beam_specifier=None,
        burnish_specifier=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        helper_functions=None,
        ):
        from abjad.tools import rhythmmakertools
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            tie_specifier=tie_specifier,
            )
        prototype = (rhythmmakertools.Talea, type(None))
        assert isinstance(talea, prototype)
        self._talea = talea
        helper_functions = helper_functions or {}
        talea_helper = helper_functions.get('talea')
        prolation_addenda_helper = helper_functions.get(
            'extra_counts_per_division')
        lefts_helper = helper_functions.get('lefts')
        middles_helper = helper_functions.get('middles')
        rights_helper = helper_functions.get('rights')
        left_lengths_helper = helper_functions.get('left_lengths')
        right_lengths_helper = helper_functions.get('right_lengths')
        secondary_divisions_helper = \
            helper_functions.get('split_divisions_by_counts')
        extra_counts_per_division = self._to_tuple(extra_counts_per_division)
        prototype = (rhythmmakertools.BurnishSpecifier, type(None))
        assert isinstance(burnish_specifier, prototype)
        self._burnish_specifier = burnish_specifier
        split_divisions_by_counts = self._to_tuple(split_divisions_by_counts)
        talea_helper = self._none_to_trivial_helper(talea_helper)
        prolation_addenda_helper = self._none_to_trivial_helper(
            prolation_addenda_helper)
        lefts_helper = self._none_to_trivial_helper(lefts_helper)
        middles_helper = self._none_to_trivial_helper(middles_helper)
        rights_helper = self._none_to_trivial_helper(rights_helper)
        left_lengths_helper = self._none_to_trivial_helper(
            left_lengths_helper)
        right_lengths_helper = self._none_to_trivial_helper(
            right_lengths_helper)
        secondary_divisions_helper = self._none_to_trivial_helper(
            secondary_divisions_helper)
        assert extra_counts_per_division is None or \
            mathtools.all_are_nonnegative_integer_equivalent_numbers(
                extra_counts_per_division)
        assert split_divisions_by_counts is None or \
            mathtools.all_are_nonnegative_integer_equivalent_numbers(
                split_divisions_by_counts)
        assert callable(talea_helper)
        assert callable(prolation_addenda_helper)
        assert callable(lefts_helper)
        assert callable(middles_helper)
        assert callable(rights_helper)
        assert callable(left_lengths_helper)
        assert callable(right_lengths_helper)
        self._extra_counts_per_division = extra_counts_per_division
        self._split_divisions_by_counts = split_divisions_by_counts
        if helper_functions == {}:
            helper_functions = None
        self._helper_functions = helper_functions

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls talea rhythm-maker on `divisions`.

        ..  container:: example

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> selections = maker(divisions)

            ::

                >>> for selection in selections:
                ...     selection
                Selection(Note("c'16"), Note("c'8"), Note("c'8."))
                Selection(Note("c'4"), Note("c'16"), Note("c'8"), Note("c'16"))
                Selection(Note("c'8"), Note("c'4"))
                Selection(Note("c'16"), Note("c'8"), Note("c'8."), Note("c'8"))

        Returns list of of selections.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    def __format__(self, format_specification=''):
        r'''Formats talea rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )

            ::

                >>> print(format(maker))
                rhythmmakertools.TaleaRhythmMaker(
                    talea=rhythmmakertools.Talea(
                        counts=(1, 2, 3, 4),
                        denominator=16,
                        ),
                    )

        Returns string.
        '''
        superclass = super(TaleaRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __illustrate__(self, divisions=None):
        r'''Illustrates talea rhythm-maker.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )
                >>> show(maker) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = maker.__illustrate__()
                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/16
                        c'8 [
                        c'16 ~ ]
                    }
                    {
                        \time 4/16
                        c'8. [
                        c'16 ]
                    }
                }

        Defaults `divisions` to ``3/8``, ``4/8``, ``3/16``, ``4/16``.

        Returns LilyPond file.
        '''
        return RhythmMaker.__illustrate__(self, divisions=divisions)

    def __repr__(self):
        r'''Gets interpreter representation of talea rhythm-maker.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )
                TaleaRhythmMaker(talea=Talea(counts=(1, 2, 3, 4), denominator=16))

        Returns string.
        '''
        return RhythmMaker.__repr__(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import rhythmmakertools
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='talea',
                command='t',
                editor=rhythmmakertools.Talea,
                ),
            systemtools.AttributeDetail(
                name='split_divisions_by_counts',
                command='sd',
                editor=idetools.getters.get_integers,
                ),
            systemtools.AttributeDetail(
                name='extra_counts_per_division',
                command='ec',
                editor=idetools.getters.get_integers,
                ),
            systemtools.AttributeDetail(
                name='beam_specifier',
                command='bs',
                editor=rhythmmakertools.BeamSpecifier,
                ),
            systemtools.AttributeDetail(
                name='burnish_specifier',
                command='us',
                editor=rhythmmakertools.BurnishSpecifier,
                ),
            systemtools.AttributeDetail(
                name='duration_spelling_specifier',
                command='ds',
                editor=rhythmmakertools.DurationSpellingSpecifier,
                ),
            systemtools.AttributeDetail(
                name='tie_specifier',
                command='ts',
                editor=rhythmmakertools.TieSpecifier,
                ),
            )

    ### PRIVATE METHODS ###

    def _add_ties(self, result, talea):
        leaves = list(iterate(result).by_class(scoretools.Leaf))
        written_durations = [leaf.written_duration for leaf in leaves]
        weights = []
        for numerator in talea:
            duration = durationtools.Duration(
                numerator,
                self.talea.denominator,
                )
            weight = abs(duration)
            weights.append(weight)
        parts = sequencetools.partition_sequence_by_weights(
            written_durations,
            weights=weights,
            allow_part_weights=More,
            cyclic=True,
            overhang=True,
            )
        counts = [len(part) for part in parts]
        parts = sequencetools.partition_sequence_by_counts(leaves, counts)
        prototype = (spannertools.Tie,)
        for part in parts:
            part = selectiontools.SliceSelection(part)
            tie_spanner = spannertools.Tie()
            # this is voodoo to temporarily neuter the contiguity constraint
            tie_spanner._contiguity_constraint = None
            for component in part:
                # TODO: make top-level detach() work here
                for spanner in component._get_spanners(prototype=prototype):
                    spanner._sever_all_components()
                #detach(prototype, component)
            # TODO: remove usage of Spanner._extend()
            tie_spanner._extend(part)

    def _burnish_all_division_parts(self, divisions, quintuplet):
        lefts, middles, rights, left_lengths, right_lengths = quintuplet
        lefts_index, rights_index = 0, 0
        burnished_divisions = []
        for division_index, division in enumerate(divisions):
            left_length = left_lengths[division_index]
            left = lefts[lefts_index:lefts_index + left_length]
            lefts_index += left_length
            right_length = right_lengths[division_index]
            right = rights[rights_index:rights_index + right_length]
            rights_index += right_length
            available_left_length = len(division)
            left_length = min([left_length, available_left_length])
            available_right_length = len(division) - left_length
            right_length = min([right_length, available_right_length])
            middle_length = len(division) - left_length - right_length
            left = left[:left_length]
            middle = middle_length * [middles[division_index]]
            right = right[:right_length]
            left_part, middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                    division,
                    [left_length, middle_length, right_length],
                    cyclic=False,
                    overhang=False,
                    )
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = left_part + middle_part + right_part
            burnished_divisions.append(burnished_division)
        unburnished_weights = [mathtools.weight(x) for x in divisions]
        burnished_weights = [mathtools.weight(x) for x in burnished_divisions]
        assert burnished_weights == unburnished_weights
        return burnished_divisions

    def _burnish_division_part(self, division_part, token):
        assert len(division_part) == len(token)
        new_division_part = []
        for number, i in zip(division_part, token):
            if i == -1:
                new_division_part.append(-abs(number))
            elif i == 0:
                new_division_part.append(number)
            elif i == 1:
                new_division_part.append(abs(number))
            else:
                raise ValueError
        new_division_part = type(division_part)(new_division_part)
        return new_division_part

    def _burnish_division_parts(self, divisions, quintuplet):
        from abjad.tools import rhythmmakertools
        burnish_specifier = self.burnish_specifier
        if burnish_specifier is None:
            burnish_specifier = rhythmmakertools.BurnishSpecifier()
        if burnish_specifier.burnish_divisions:
            return self._burnish_all_division_parts(divisions, quintuplet)
        elif burnish_specifier.burnish_output:
            return self._burnish_first_and_last_division_parts(
                divisions, quintuplet)
        else:
            return divisions

    def _burnish_first_and_last_division_parts(self, divisions, quintuplet):
        lefts, middles, rights, left_lengths, right_lengths = quintuplet
        burnished_divisions = []
        left_length = left_lengths[0]
        left = lefts[:left_length]
        right_length = right_lengths[0]
        right = rights[:right_length]
        if len(divisions) == 1:
            available_left_length = len(divisions[0])
            left_length = min([left_length, available_left_length])
            available_right_length = len(divisions[0]) - left_length
            right_length = min([right_length, available_right_length])
            middle_length = len(divisions[0]) - left_length - right_length
            left = left[:left_length]
            middle = middle_length * [middles[0]]
            right = right[:right_length]
            left_part, middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                    divisions[0],
                    [left_length, middle_length, right_length],
                    cyclic=False,
                    overhang=False,
                    )
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = left_part + middle_part + right_part
            burnished_divisions.append(burnished_division)
        else:
            # first division
            available_left_length = len(divisions[0])
            left_length = min([left_length, available_left_length])
            middle_length = len(divisions[0]) - left_length
            left = left[:left_length]
            middle = middle_length * [middles[0]]
            left_part, middle_part = \
                sequencetools.partition_sequence_by_counts(
                    divisions[0],
                    [left_length, middle_length],
                    cyclic=False,
                    overhang=False,
                    )
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            burnished_division = left_part + middle_part
            burnished_divisions.append(burnished_division)
            # middle divisions
            for division in divisions[1:-1]:
                middle_part = division
                middle = len(division) * [middles[0]]
                middle_part = self._burnish_division_part(middle_part, middle)
                burnished_division = middle_part
                burnished_divisions.append(burnished_division)
            # last division:
            available_right_length = len(divisions[-1])
            right_length = min([right_length, available_right_length])
            middle_length = len(divisions[-1]) - right_length
            right = right[:right_length]
            middle = middle_length * [middles[0]]
            middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                    divisions[-1],
                    [middle_length, right_length],
                    cyclic=False,
                    overhang=False,
                    )
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = middle_part + right_part
            burnished_divisions.append(burnished_division)
        unburnished_weights = [mathtools.weight(x) for x in divisions]
        burnished_weights = [mathtools.weight(x) for x in burnished_divisions]
        assert burnished_weights == unburnished_weights
        return burnished_divisions

    def _make_leaf_lists(self, numeric_map, talea_denominator):
        from abjad.tools import rhythmmakertools
        leaf_lists = []
        specifier = self.duration_spelling_specifier
        if specifier is None:
            specifier = rhythmmakertools.DurationSpellingSpecifier()
        for map_division in numeric_map:
            leaf_list = scoretools.make_leaves_from_talea(
                map_division,
                talea_denominator,
                decrease_durations_monotonically=\
                    specifier.decrease_durations_monotonically,
                forbidden_written_duration=\
                    specifier.forbidden_written_duration,
                )
            leaf_lists.append(leaf_list)
        return leaf_lists

    def _make_music(self, duration_pairs, seeds):
        from abjad.tools import rhythmmakertools
        octuplet = self._prepare_input(seeds)
        talea, extra_counts_per_division = octuplet[:2]
        unscaled_talea = tuple(talea)
        split_divisions_by_counts = octuplet[-1]
        taleas = (talea, extra_counts_per_division, split_divisions_by_counts)
        result = self._scale_taleas(
            duration_pairs, self.talea.denominator, taleas)
        duration_pairs, lcd, talea, extra_counts_per_division, split_divisions_by_counts = \
            result
        secondary_duration_pairs = self._make_secondary_duration_pairs(
            duration_pairs, split_divisions_by_counts)
        septuplet = (talea, extra_counts_per_division) + octuplet[2:-1]
        numeric_map = self._make_numeric_map(
            secondary_duration_pairs, septuplet)
        leaf_lists = self._make_leaf_lists(numeric_map, lcd)
        if not extra_counts_per_division:
            result = leaf_lists
        else:
            tuplets = self._make_tuplets(secondary_duration_pairs, leaf_lists)
            result = tuplets
        # maybe hoist up to RhythmMaker
        beam_specifier = self.beam_specifier
        if beam_specifier is None:
            beam_specifier = rhythmmakertools.BeamSpecifier()
        if beam_specifier.beam_divisions_together:
            durations = []
            for x in result:
                if isinstance(x, selectiontools.Selection):
                    duration = x.get_duration()
                else:
                    duration = x._get_duration()
                durations.append(duration)
            beam = spannertools.DuratedComplexBeam(
                durations=durations,
                span_beam_count=1,
                )
            components = []
            for x in result:
                if isinstance(x, selectiontools.Selection):
                    components.extend(x)
                elif isinstance(x, scoretools.Tuplet):
                    components.append(x)
                else:
                    raise TypeError(x)
            attach(beam, components)
        elif beam_specifier.beam_each_division:
            for cell in result:
                beam = spannertools.MultipartBeam()
                attach(beam, cell)
        tie_specifier = self.tie_specifier
        if tie_specifier is None:
            tie_specifier = rhythmmakertools.TieSpecifier()
        if tie_specifier.tie_split_notes:
            self._add_ties(result, unscaled_talea)
        result = [selectiontools.Selection(x) for x in result]
        return result

    def _make_numeric_map(self, duration_pairs, septuplet):
        talea, extra_counts_per_division, lefts, middles, rights, left_lengths, right_lengths = septuplet
        prolated_duration_pairs = self._make_prolated_duration_pairs(
            duration_pairs, extra_counts_per_division)
        if isinstance(prolated_duration_pairs[0], tuple):
            prolated_numerators = [
                pair[0] for pair in prolated_duration_pairs]
        else:
            prolated_numerators = [
                pair.numerator for pair in prolated_duration_pairs]
        map_divisions = self._split_sequence_extended_to_weights(
            talea, prolated_numerators, overhang=False)
        quintuplet = (lefts, middles, rights, left_lengths, right_lengths)
        burnished_map_divisions = self._burnish_division_parts(
            map_divisions, quintuplet)
        numeric_map = burnished_map_divisions
        return numeric_map

    def _make_prolated_duration_pairs(self, duration_pairs, extra_counts_per_division):
        prolated_duration_pairs = []
        for i, duration_pair in enumerate(duration_pairs):
            if not extra_counts_per_division:
                prolated_duration_pairs.append(duration_pair)
            else:
                prolation_addendum = extra_counts_per_division[i]
                if hasattr(duration_pair, 'numerator'):
                    prolation_addendum %= duration_pair.numerator
                else:
                    prolation_addendum %= duration_pair[0]
                if isinstance(duration_pair, tuple):
                    numerator, denominator = duration_pair
                else:
                    numerator, denominator = duration_pair.pair
                prolated_duration_pair = (
                    numerator + prolation_addendum,
                    denominator,
                    )
                prolated_duration_pairs.append(prolated_duration_pair)
        return prolated_duration_pairs

    def _prepare_input(self, seeds):
        from abjad.tools import rhythmmakertools
        helper_functions = self.helper_functions or {}
        talea = self.talea.counts or ()
        talea_helper = self._none_to_trivial_helper(
            helper_functions.get('talea'))
        talea = talea_helper(talea, seeds)
        talea = datastructuretools.CyclicTuple(talea)

        extra_counts_per_division = self.extra_counts_per_division or ()
        prolation_addenda_helper = self._none_to_trivial_helper(
            helper_functions.get('extra_counts_per_division'))
        extra_counts_per_division = prolation_addenda_helper(
            extra_counts_per_division, seeds)
        extra_counts_per_division = datastructuretools.CyclicTuple(
            extra_counts_per_division)

        burnish_specifier = self.burnish_specifier
        if burnish_specifier is None:
            burnish_specifier = rhythmmakertools.BurnishSpecifier()

        lefts = burnish_specifier.lefts or ()
        lefts_helper = self._none_to_trivial_helper(
            helper_functions.get('lefts'))
        lefts = lefts_helper(lefts, seeds)
        lefts = datastructuretools.CyclicTuple(lefts)

        middles = burnish_specifier.middles or ()
        middles_helper = self._none_to_trivial_helper(
            helper_functions.get('middles'))
        middles = middles_helper(middles, seeds)
        middles = datastructuretools.CyclicTuple(middles)

        rights = burnish_specifier.rights or ()
        rights_helper = self._none_to_trivial_helper(
            helper_functions.get('rights'))
        rights = rights_helper(rights, seeds)
        rights = datastructuretools.CyclicTuple(rights)

        left_lengths = burnish_specifier.left_lengths or ()
        left_lengths_helper = self._none_to_trivial_helper(
            helper_functions.get('left_lengths'))
        left_lengths = left_lengths_helper(left_lengths, seeds)
        left_lengths = datastructuretools.CyclicTuple(left_lengths)

        right_lengths = burnish_specifier.right_lengths or ()
        right_lengths_helper = self._none_to_trivial_helper(
            helper_functions.get('right_lengths'))
        right_lengths = right_lengths_helper(right_lengths, seeds)
        right_lengths = datastructuretools.CyclicTuple(right_lengths)

        split_divisions_by_counts = self.split_divisions_by_counts or ()
        secondary_divisions_helper = self._none_to_trivial_helper(
            helper_functions.get('split_divisions_by_counts'))
        split_divisions_by_counts = secondary_divisions_helper(
            split_divisions_by_counts, seeds)
        split_divisions_by_counts = datastructuretools.CyclicTuple(
            split_divisions_by_counts)

        return (
            talea,
            extra_counts_per_division,
            lefts,
            middles,
            rights,
            left_lengths,
            right_lengths,
            split_divisions_by_counts,
            )

    @staticmethod
    def _split_sequence_extended_to_weights(sequence, weights, overhang=True):
        n = int(
            math.ceil(float(mathtools.weight(weights)) / 
            mathtools.weight(sequence))
            )
        sequence = sequencetools.repeat_sequence(sequence, n)
        return sequencetools.split_sequence(
            sequence, 
            weights, 
            cyclic=False, 
            overhang=overhang,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def beam_specifier(self):
        r'''Gets beam specifier of talea rhythm-maker.

        Three beam specifier configurations are available.

        ..  container:: example

            This rhythm-maker beams each division:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1,),
                ...     denominator=16,
                ...     )
                >>> beam_specifier = rhythmmakertools.BeamSpecifier(
                ...     beam_each_division=True,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     beam_specifier=beam_specifier,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }

            The behavior shown here is the talea rhythm-maker's default
            beaming.

        ..  container:: example

            This rhythm-maker beams divisions together:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1,),
                ...     denominator=16,
                ...     )
                >>> beam_specifier = rhythmmakertools.BeamSpecifier(
                ...     beam_divisions_together=True,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     beam_specifier=beam_specifier,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        \set stemLeftBeamCount = #0
                        \set stemRightBeamCount = #2
                        c'16 [
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #1
                        c'16
                    }
                    {
                        \time 4/8
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #1
                        c'16
                    }
                    {
                        \time 3/8
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #1
                        c'16
                    }
                    {
                        \time 4/8
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #2
                        c'16
                        \set stemLeftBeamCount = #2
                        \set stemRightBeamCount = #0
                        c'16 ]
                    }
                }

        ..  container:: example

            This rhythm-maker makes no beams:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1,),
                ...     denominator=16,
                ...     )
                >>> beam_specifier = rhythmmakertools.BeamSpecifier(
                ...     beam_each_division=False,
                ...     beam_divisions_together=False,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     beam_specifier=beam_specifier,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                    {
                        \time 4/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                    {
                        \time 3/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                    {
                        \time 4/8
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                    }
                }


        Returns beam specifier or none.
        '''
        return RhythmMaker.beam_specifier.fget(self)

    @property
    def burnish_specifier(self):
        r'''Gets burnish specifier of talea rhythm-maker.

        ..  container:: example

            'Output burnishing' means forcibly to cast the first leaf (or
            leaves) of output; or forcibly to cast the last leaf (or leaves) of
            output; or to cast both the first and last leaves of output at the
            same time.

            This example makes a talea rhythm with the first leaf of output
            forcibly cast to a rest and also with the last two leaves of output
            forcibly cast to rests:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     burnish_output=True,
                ...     lefts=(-1,),
                ...     middles=(0,),
                ...     rights=(-1,),
                ...     left_lengths=(1,),
                ...     right_lengths=(2,),
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     burnish_specifier=burnish_specifier,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        r16
                        c'8 [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8 ]
                        r8.
                        r8
                    }
                }

        ..  container:: example

            'Division burnishing' means forcibly to cast the first leaf (or
            leaves) of every division; or forcibly to cast the last
            leaf (or leaves) of every division; or forcibly to cast
            both the first and last leaves of every division at the same time.

            This example makes a talea rhythm with the first leaf of every
            division forcibly cast to a rest:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     burnish_divisions=True,
                ...     lefts=(-1,),
                ...     middles=(0,),
                ...     rights=(0,),
                ...     left_lengths=(1,),
                ...     right_lengths=(0,),
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     burnish_specifier=burnish_specifier,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        r16
                        c'8 [
                        c'8. ]
                    }
                    {
                        \time 4/8
                        r4
                        c'16 [
                        c'8
                        c'16 ]
                    }
                    {
                        \time 3/8
                        r8
                        c'4
                    }
                    {
                        \time 4/8
                        r16
                        c'8 [
                        c'8.
                        c'8 ]
                    }
                }

        Returns burnish specifier or none.
        '''
        return self._burnish_specifier

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier of talea rhythm-maker.

        Several beam spelling specifier configurations are available.

        ..  container:: example

            This rhythm-maker spells nonassignable durations like ``5/16`` with
            monontonically decreasing durations:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(5,),
                ...     denominator=16,
                ...     )
                >>> duration_spelling_specifier = \
                ...     rhythmmakertools.DurationSpellingSpecifier(
                ...     decrease_durations_monotonically=True,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     beam_specifier=beam_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (5, 8), (5, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'4 ~
                        c'16
                        c'4 ~
                        c'16
                    }
                    {
                        c'4 ~
                        c'16
                        c'4 ~
                        c'16
                    }
                    {
                        c'4 ~
                        c'16
                        c'4 ~
                        c'16
                    }
                }

            The behavior shown here is a default duration-spelling behavior.

        ..  container:: example

            This rhythm-maker spells nonassignable durations like ``5/16`` with
            monontonically increasing durations:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(5,),
                ...     denominator=16,
                ...     )
                >>> duration_spelling_specifier = \
                ...     rhythmmakertools.DurationSpellingSpecifier(
                ...     decrease_durations_monotonically=False,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     duration_spelling_specifier=duration_spelling_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (5, 8), (5, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'16 ~
                        c'4
                        c'16 ~
                        c'4
                    }
                    {
                        c'16 ~
                        c'4
                        c'16 ~
                        c'4
                    }
                    {
                        c'16 ~
                        c'4
                        c'16 ~
                        c'4
                    }
                }

        ..  container:: example

            This rhythm-maker has no forbidden durations:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 1, 1, 1, 4, 4),
                ...     denominator=16,
                ...     )
                >>> duration_spelling_specifier = \
                ...     rhythmmakertools.DurationSpellingSpecifier(
                ...     forbidden_written_duration=None,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     duration_spelling_specifier=duration_spelling_specifier,
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'16 [
                        c'16
                        c'16
                        c'16 ]
                        c'4
                        c'4
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16 ]
                        c'4
                        c'4
                    }
                }

            The behavior shown here is a default duration-spelling behavior.

        ..  container:: example

            This rhythm-maker forbids durations equal to ``1/4`` or greater:

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 1, 1, 1, 4, 4),
                ...     denominator=16,
                ...     )
                >>> duration_spelling_specifier = \
                ...     rhythmmakertools.DurationSpellingSpecifier(
                ...     forbidden_written_duration=Duration(1, 4),
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     duration_spelling_specifier=duration_spelling_specifier,
                ...     )

            ::

                >>> divisions = [(3, 4), (3, 4)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'8 ~
                        c'8
                        c'8 ~
                        c'8 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'8 ~
                        c'8
                        c'8 ~
                        c'8 ]
                    }
                }

            Forbidden durations are rewritten with smaller durations tied
            together.

        Returns duration spelling specifier or none.
        '''
        return RhythmMaker.duration_spelling_specifier.fget(self)

    @property
    def extra_counts_per_division(self):
        r'''Gets extra counts per division of talea rhythm-maker.

        ..  container:: example

            Here's a talea:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8
                        c'8.
                        c'8 ]
                    }
                }

            Here's the same rhythm with an extra count added to every other
            division:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     extra_counts_per_division=(0, 1,),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \times 8/9 {
                            c'4
                            c'16 [
                            c'8
                            c'8 ~ ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'16
                            c'4
                            c'16
                        }
                    }
                    {
                        \time 4/8
                        \times 8/9 {
                            c'8 [
                            c'8. ]
                            c'4
                        }
                    }
                }

            And here's the same rhythm with two extra counts added to every
            other division:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     extra_counts_per_division=(0, 2,),
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 4/8
                        \times 4/5 {
                            c'4
                            c'16 [
                            c'8
                            c'8. ]
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'4
                            c'16 [
                            c'16 ~ ]
                        }
                    }
                    {
                        \time 4/8
                        \times 4/5 {
                            c'16 [
                            c'8. ]
                            c'4
                            c'16 [
                            c'16 ]
                        }
                    }
                }

            Note that the duration of each added count is equal to the duration
            of each count in the rhythm-maker's input talea.

        Returns integer tuple or none.
        '''
        return self._extra_counts_per_division

    @property
    def helper_functions(self):
        r'''Gets helper functions of talea rhythm-maker.

        Returns dictionary or none.
        '''
        return self._helper_functions

    @property
    def split_divisions_by_counts(self):
        r'''Gets secondary divisions of talea rhythm-maker.

        Secondary divisions impose a cyclic split operation on divisions.

        ..  container:: example

            Here's a talea equal to two thirty-second repeating indefinitely.
            The maker makes four divisions equal to 12 thirty-second notes
            each:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(2,),
                ...     denominator=32,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }

            Here's the same talea with secondary divisions set to split the
            divisions every 17 thirty-second notes. The maker makes six
            divisions with durations equal, respectively, to 12, 5, 7, 10, 2
            and 12 thirty-second notes.

            Note that ``12 + 5 = 17`` and ``7 + 10 = 17``:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(2,),
                ...     denominator=32,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     split_divisions_by_counts=(17,)
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'32 ~ ]
                        c'32 [
                        c'16
                        c'16
                        c'16 ]
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16 ]
                        c'16
                    }
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }

            Note that the additional divisions created when using
            `split_divisions_by_counts` are subject to `extra_counts_per_division` just like
            other divisions.

            This example adds one extra thirty-second note to every other
            division. The durations of the divisions remain the same as in the
            previous example. But now every other division is tupletted:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(2,),
                ...     denominator=32,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     split_divisions_by_counts=(17,),
                ...     extra_counts_per_division=(0, 1),
                ...     )

            ::

                >>> divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                    }
                    {
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16 [
                            c'16
                            c'16 ]
                        }
                        {
                            c'16 [
                            c'16
                            c'16
                            c'32 ~ ]
                        }
                    }
                    {
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 10/11 {
                            c'32 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16 ]
                        }
                        {
                            c'16
                        }
                    }
                    {
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 12/13 {
                            c'16 [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'32 ]
                        }
                    }
                }

        Returns positive integer tuple or none.
        '''
        return self._split_divisions_by_counts

    @property
    def talea(self):
        r'''Gets talea of talea rhythm-maker.

        Returns tuple.
        '''
        return self._talea

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses talea rhythm-maker.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...     counts=(1, 2, 3, 4),
                ...     denominator=16,
                ...     )
                >>> maker = rhythmmakertools.TaleaRhythmMaker(
                ...     talea=talea,
                ...     )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'16 [
                        c'8
                        c'8. ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'16 [
                        c'8
                        c'16 ~ ]
                    }
                    {
                        \time 3/8
                        c'8
                        c'4
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8
                        c'8.
                        c'8 ]
                    }
                }

            ::

                >>> reversed_maker = maker.reverse()
                >>> print(format(reversed_maker))
                rhythmmakertools.TaleaRhythmMaker(
                    talea=rhythmmakertools.Talea(
                        counts=(4, 3, 2, 1),
                        denominator=16,
                        ),
                    burnish_specifier=rhythmmakertools.BurnishSpecifier(
                        burnish_divisions=False,
                        burnish_output=False,
                        ),
                    duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                        decrease_durations_monotonically=False,
                        ),
                    )

            ::

                >>> divisions = [(3, 8), (4, 8), (3, 8), (4, 8)]
                >>> music = reversed_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 3/8
                        c'4
                        c'8 ~
                    }
                    {
                        \time 4/8
                        c'16 [
                        c'8
                        c'16 ]
                        c'4
                    }
                    {
                        \time 3/8
                        c'8. [
                        c'8
                        c'16 ]
                    }
                    {
                        \time 4/8
                        c'4
                        c'8. [
                        c'16 ]
                    }
                }

        Defined equal to copy of this talea rhythm-maker with `talea`,
        `extra_counts_per_division`, `split_divisions_by_counts`,
        `burnish_specifier` and `duration_spelling_specifier` reversed.

        Returns new talea rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        talea = self.talea.reverse()
        extra_counts_per_division = \
            self._reverse_tuple(self.extra_counts_per_division)
        burnish_specifier = self.burnish_specifier
        if burnish_specifier is None:
            burnish_specifier = rhythmmakertools.BurnishSpecifier()
        burnish_specifier = burnish_specifier.reverse()
        split_divisions_by_counts = \
            self._reverse_tuple(self.split_divisions_by_counts)
        duration_spelling_specifier = self.duration_spelling_specifier
        if duration_spelling_specifier is None:
            duration_spelling_specifier = \
                rhythmmakertools.DurationSpellingSpecifier()
        duration_spelling_specifier = duration_spelling_specifier.reverse()
        maker = new(
            self,
            burnish_specifier=burnish_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            extra_counts_per_division=extra_counts_per_division,
            split_divisions_by_counts=split_divisions_by_counts,
            talea=talea,
            )
        return maker

    def rotate(self, n=0):
        r'''Rotates talea rhythm-maker.

        Returns newly constructed rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        talea = self.talea.rotate(n)
        extra_counts_per_division = \
            self._rotate_tuple(self.extra_counts_per_division, n)
        burnish_specifier = self.burnish_specifier
        if burnish_specifier is None:
            burnish_specifier = rhythmmakertools.BurnishSpecifier()
        burnish_specifier = burnish_specifier.rotate(n)
        split_divisions_by_counts = \
            self._rotate_tuple(self.split_divisions_by_counts, n)
        maker = new(
            self,
            burnish_specifier=burnish_specifier,
            extra_counts_per_division=extra_counts_per_division,
            split_divisions_by_counts=split_divisions_by_counts,
            talea=talea,
            )
        return maker


# -*- encoding: utf-8 -*-
import numbers
from abjad.tools import mathtools
from abjad.tools.pitchtools.PitchClass import PitchClass


class NumberedPitchClass(PitchClass):
    '''A numbered pitch-class.

    ::

        >>> pitchtools.NumberedPitchClass(13)
        NumberedPitchClass(1)

    ::

        >>> pitchtools.NumberedPitchClass('d')
        NumberedPitchClass(2)

    ::

        >>> pitchtools.NumberedPitchClass(NamedPitch('g,'))
        NumberedPitchClass(7)

    ::

        >>> pitchtools.NumberedPitchClass(pitchtools.NumberedPitch(15))
        NumberedPitchClass(3)

    ::

        >>> pitchtools.NumberedPitchClass(pitchtools.NamedPitchClass('e'))
        NumberedPitchClass(4)

    ::

        >>> pitchtools.NumberedPitchClass('C#5')
        NumberedPitchClass(1)

    ::

        >>> pitchtools.NumberedPitchClass(Note("a'8."))
        NumberedPitchClass(9)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pitch_class_number',
        )

    ### INITIALIZER ###

    def __init__(self, expr=None):
        from abjad.tools import pitchtools
        prototype = (numbers.Number, pitchtools.NumberedPitch, type(self))
        if isinstance(expr, prototype):
            self._initialize_by_number(float(expr))
        elif isinstance(expr, pitchtools.NamedPitch):
            self._initialize_by_named_pitch(expr)
        elif isinstance(expr, pitchtools.NamedPitchClass):
            self._initialize_by_named_pitch_class(expr)
        elif isinstance(expr, str):
            self._initialize_by_string(expr)
        elif pitchtools.Pitch.is_pitch_carrier(expr):
            self._initialize_by_pitch_carrier(expr)
        elif expr is None:
            self._initialize_by_number(0)
        else:
            message = 'can not instantiate {}: {!r}.'
            message = message.format(type(self).__name__, expr)
            raise TypeError(message)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Adds `expr` to numbered pitch-class.

        ::

            >>> pitch_class = pitchtools.NumberedPitchClass(9)
            >>> interval = pitchtools.NumberedInterval(4)
            >>> pitch_class + interval
            NumberedPitchClass(1)

        Returns new numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        interval = pitchtools.NumberedInterval(expr)
        return type(self)(self.pitch_class_number + interval.number % 12)

    def __copy__(self, *args):
        r'''Copies numbered pitch-class.

        ::

            >>> import copy
            >>> pitch_class = pitchtools.NumberedPitchClass(9)
            >>> copy.copy(pitch_class)
            NumberedPitchClass(9)

        Returns new numbered pitch-class.
        '''
        return type(self)(self)

    def __eq__(self, arg):
        r'''Is true when `arg` is a numbered pitch-class with pitch-class number
        equal to that of this numbered pitch-class.

        ::

            >>> pitch_class_1 = pitchtools.NumberedPitchClass(9)
            >>> pitch_class_2 = pitchtools.NumberedPitchClass(3)
            >>> pitch_class_1 == pitch_class_1
            True

        Otherwise false:

        ::

            >>> pitch_class_1 == pitch_class_2
            False

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self._pitch_class_number == \
                arg._pitch_class_number
        return self._pitch_class_number == arg

    def __float__(self):
        r'''Changes numbered pitch-class to float.

        ::

            >>> pitch_class = pitchtools.NumberedPitchClass(9)
            >>> float(pitch_class)
            9.0

        Returns float.
        '''
        return float(self._pitch_class_number)

    def __hash__(self):
        r'''Hashes numbered pitch-class.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NumberedPitchClass, self).__hash__()

    def __int__(self):
        r'''Changes numbered pitch-class to integer.

        ::

            >>> pitch_class = pitchtools.NumberedPitchClass(9)
            >>> int(pitch_class)
            9

        Returns integer.
        '''
        return self._pitch_class_number

    def __neg__(self):
        r'''Negates numbered pitch-class.

        ::

            >>> pitch_class = pitchtools.NumberedPitchClass(9)
            >>> -pitch_class
            NumberedPitchClass(3)

        Returns new numbered pitch-class.
        '''
        return type(self)(-self.pitch_class_number)

    def __str__(self):
        r'''String representation of numbered pitch-class.

        Returns string.
        '''
        return str(self.pitch_class_number)

    def __sub__(self, expr):
        r'''Subtracts `expr` from numbered pitch-class.

        Subtraction defined against both numbered intervals
        and against other pitch-classes.

        Returns numbered inversion-equivalent interval-class.
        '''
        from abjad.tools import pitchtools
        if isinstance(expr, type(self)):
            interval_class_number = abs(
                self.pitch_class_number -
                expr.pitch_class_number
                )
            if 6 < interval_class_number:
                interval_class_number = 12 - interval_class_number
            return pitchtools.NumberedInversionEquivalentIntervalClass(
                interval_class_number)
        interval_class = pitchtools.NumberedInversionEquivalentIntervalClass(
            expr)
        return type(self)(self.pitch_class_number - interval_class.number % 12)

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.pitch_class_number,
            )
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PRIVATE METHODS ###

    def _initialize_by_named_pitch(self, expr):
        self._pitch_class_number = expr.pitch_class_number

    def _initialize_by_named_pitch_class(self, expr):
        self._pitch_class_number = expr.pitch_class_number

    def _initialize_by_number(self, expr):
        self._pitch_class_number = \
            mathtools.integer_equivalent_number_to_integer(
                round((float(expr) % 12) * 2) / 2)

    def _initialize_by_pitch_carrier(self, expr):
        from abjad.tools import pitchtools
        named_pitch = pitchtools.get_named_pitch_from_pitch_carrier(
            expr)
        self._initialize_by_named_pitch(named_pitch)

    def _initialize_by_string(self, expr):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(expr)
        self._initialize_by_named_pitch_class(named_pitch_class)

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental=None):
        '''Applies `accidental` to numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(1).apply_accidental('flat')
            NumberedPitchClass(0)

        Returns new numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        accidental = pitchtools.Accidental(accidental)
        semitones = self.pitch_class_number + accidental.semitones
        return type(self)(semitones)

    def invert(self):
        r'''Invertes numbered pitch-class.

        Returns new numbered pitch-class.
        '''
        return type(self)(12 - self.pitch_class_number)

    def multiply(self, n=1):
        r'''Multiplies pitch-class number by `n`.

        ::

            >>> pitchtools.NumberedPitchClass(11).multiply(3)
            NumberedPitchClass(9)

        Returns new numbered pitch-class.
        '''
        return type(self)(self.pitch_class_number * n)

    def transpose(self, n):
        r'''Transposes numbered pitch-class by `n` semitones.

        Returns new numbered pitch-class.
        '''
        return type(self)(self.pitch_class_number + n)

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Accidental of numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(1).accidental
            Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self.alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Alteration of numbered pitch-class in semitones.

        ::

            >>> pitchtools.NumberedPitchClass(1).alteration_in_semitones
            1

        ::

            >>> pitchtools.NumberedPitchClass(10.5).alteration_in_semitones
            -0.5

        Returns integer or float.
        '''
        return (self.pitch_class_number -
            self._diatonic_pitch_class_number_to_pitch_class_number[
                self.diatonic_pitch_class_number])

    @property
    def diatonic_pitch_class_name(self):
        r'''Diatonic pitch-class name corresponding to numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(1).diatonic_pitch_class_name
            'c'

        Returns string.
        '''
        return self.pitch_class_name[0]

    @property
    def diatonic_pitch_class_number(self):
        r'''Diatonic pitch-class number corresponding to numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(1).diatonic_pitch_class_number
            0

        Returns integer.
        '''
        return self._diatonic_pitch_class_name_to_diatonic_pitch_class_number[
            self.diatonic_pitch_class_name]

    @property
    def named_pitch_class(self):
        r'''Named pitch-class corresponding to numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(13).named_pitch_class
            NamedPitchClass('cs')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def numbered_pitch_class(self):
        r'''Numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(13).numbered_pitch_class
            NumberedPitchClass(1)

        Returns new numbered pitch-class.
        '''
        return type(self)(self)

    @property
    def pitch_class_label(self):
        r'''Pitch-class / octave label of numbered pitch-class.

        ::

            >>> pitchtools.NumberedPitchClass(13).pitch_class_label
            'C#'

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            )

    @property
    def pitch_class_name(self):
        r'''Pitch-class name.

        ::

            >>> pitchtools.NumberedPitchClass(1).pitch_class_name
            'cs'

        Returns string.
        '''
        if self.accidental_spelling == 'mixed':
            return self._pitch_class_number_to_pitch_class_name[
                self._pitch_class_number]
        elif self.accidental_spelling == 'sharps':
            return self._pitch_class_number_to_pitch_class_name_with_sharps[
                self._pitch_class_number]
        elif self.accidental_spelling == 'flats':
            return self._pitch_class_number_to_pitch_class_name_with_flats[
                self._pitch_class_number]
        else:
            raise ValueError

    @property
    def pitch_class_number(self):
        r'''Pitch-class number.

        ::

            >>> pitchtools.NumberedPitchClass(1).pitch_class_number
            1

        Returns number.
        '''
        return self._pitch_class_number


# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker___eq___01():

    talea = rhythmmakertools.Talea(
        counts=(-1, 2, -3, 4),
        denominator=16,
        )
    maker_1 = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=(2, 3),
        split_divisions_by_counts=(6,),
        )

    talea = rhythmmakertools.Talea(
        counts=(-1, 2, -3, 4),
        denominator=16,
        )
    maker_2 = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=(2, 3),
        split_divisions_by_counts=(6,),
        )

    assert maker_1 == maker_1
    assert maker_1 == maker_2
    assert not maker_1 == 'foo'
    assert maker_2 == maker_1
    assert maker_2 == maker_2
    assert not maker_2 == 'foo'
    assert not 'foo' == maker_1
    assert not 'foo' == maker_2
    assert 'foo' == 'foo'


# -*- encoding: utf-8 -*-
import collections
from abjad.tools.abctools import AbjadObject


class Sequence(AbjadObject):
    r'''A sequence.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_elements',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        args = args or ()
        elements = tuple(args)
        self._elements = elements

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Adds `expr` to sequence.

        ..  container:: example

            Adds sequence to sequence:

            ::

                >>> Sequence(1, 2, 3) + Sequence(4, 5, 6)
                Sequence(1, 2, 3, 4, 5, 6)

            Adds tuple to sequence:

            ::

                >>> Sequence(1, 2, 3) + (4, 5, 6)
                Sequence(1, 2, 3, 4, 5, 6)

            Adds list to sequence:

            ::

                >>> Sequence(1, 2, 3) + [4, 5, 6]
                Sequence(1, 2, 3, 4, 5, 6)

        Returns new sequence.
        '''
        expr = type(self)(*expr)
        elements = self._elements + expr._elements
        return type(self)(*elements)

    def __eq__(self, expr):
        r'''Is true when `expr` is a sequence with elements equal to those of
        this sequence. Otherwise false.

        ..  container:: example

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6) == Sequence(1, 2, 3, 4, 5, 6)
                True

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6) == (1, 2, 3, 4, 5, 6)
                False

        '''
        if isinstance(expr, type(self)):
            return self._elements == expr._elements
        return False

    def __format__(self, format_specification=''):
        r'''Formats sequence.

        ..  container:: example

            Prints format:

            ::

                >>> print(format(Sequence(1, 2, 3, 4, 5, 6)))
                sequencetools.Sequence(1, 2, 3, 4, 5, 6)

        Returns string.
        '''
        return AbjadObject.__format__(
            self,
            format_specification=format_specification,
            )

    def __getitem__(self, i):
        r'''Gets item `i` from sequence.

        ..  container:: example

            Gets last item in sequence:

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6)[-1]
                6

        Returns item.
        '''
        result = self._elements.__getitem__(i)
        if isinstance(i, slice):
            return type(self)(*result)
        return result

    def __getslice__(self, start, stop):
        r'''Gets slice from `start` to `stop`.

        ..  container:: example

            Gets last three items in sequence:

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6)[-3:]
                Sequence(4, 5, 6)

        Returns new sequence.
        '''
        result = self._elements.__getslice__(start, stop)
        result = type(self)(*result)
        return result

    def __hash__(self):
        r'''Hashes sequence.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Sequence, self).__hash__()

    def __len__(self):
        r'''Gets length of sequence.

        ..  container:: example

            Gets length of six-item sequence:

            ::

                >>> len(Sequence(1, 2, 3, 4, 5, 6))
                6

        Returns nonnegative integer.
        '''
        return len(self._elements)

    def __ne__(self, expr):
        r'''Is true when sequence is not equal to `expr`. Otherwise false.

        ..  container:: example

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6) != (1, 2, 3, 4, 5, 6)
                True

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6) != Sequence(1, 2, 3, 4, 5, 6)
                False

        '''
        return not self == expr

    def __radd__(self, expr):
        r'''Adds sequence to `expr`.

        ..  container:: example

            Adds sequence to sequence:

            ::

                >>> Sequence(1, 2, 3) + Sequence(4, 5, 6)
                Sequence(1, 2, 3, 4, 5, 6)

            Adds sequence to tuple:

            ::

                >>> (1, 2, 3) + Sequence(4, 5, 6)
                Sequence(1, 2, 3, 4, 5, 6)

            Adds sequence to list:

            ::

                >>> [1, 2, 3] + Sequence(4, 5, 6)
                Sequence(1, 2, 3, 4, 5, 6)

        Returns new sequence.
        '''
        expr = type(self)(*expr)
        elements = expr._elements + self._elements
        return type(self)(*elements)

    def __repr__(self):
        r'''Gets interpreter representation of sequence.

        ..  container:: example

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6)
                Sequence(1, 2, 3, 4, 5, 6)

        Returns string.
        '''
        return AbjadObject.__repr__(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = tuple(self._elements)
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def degree_of_rotational_symmetry(self):
        '''Gets degree of rotational symmetry.

        ..  container:: example

            ::

                >>> Sequence(1, 1, 1, 1, 1, 1).degree_of_rotational_symmetry
                6

            ::

                >>> Sequence(1, 2, 1, 2, 1, 2).degree_of_rotational_symmetry
                3

            ::

                >>> Sequence(1, 2, 3, 1, 2, 3).degree_of_rotational_symmetry
                2

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6).degree_of_rotational_symmetry
                1

            ::

                >>> Sequence().degree_of_rotational_symmetry
                1

        Returns positive integer.
        '''
        degree_of_rotational_symmetry = 0
        for index in range(len(self)):
            rotation = self[index:] + self[:index]
            if rotation == self:
                degree_of_rotational_symmetry += 1
        degree_of_rotational_symmetry = degree_of_rotational_symmetry or 1
        return degree_of_rotational_symmetry

    @property
    def period_of_rotation(self):
        '''Gets period of rotation.

        ..  container:: example

            ::

                >>> Sequence(1, 2, 3, 4, 5, 6).period_of_rotation
                6

            ::

                >>> Sequence(1, 2, 3, 1, 2, 3).period_of_rotation
                3

            ::

                >>> Sequence(1, 2, 1, 2, 1, 2).period_of_rotation
                2

            ::

                >>> Sequence(1, 1, 1, 1, 1, 1).period_of_rotation
                1

            ::

                >>> Sequence().period_of_rotation
                0

        Defined equal to length of sequence divided by degree of rotational
        symmetry of sequence.

        Returns positive integer.
        '''
        return len(self) // self.degree_of_rotational_symmetry

    ### PUBLIC METHODS ###

    def is_decreasing(self, strict=True):
        r'''Is true when sequence decreases.

        ..  container:: example

            With ``strict=True``:

            ::

                >>> Sequence(5, 4, 3, 2, 1, 0).is_decreasing(strict=True)
                True

            ::

                >>> Sequence(3, 3, 3, 2, 1, 0).is_decreasing(strict=True)
                False

            ::

                >>> Sequence(3, 3, 3, 3, 3, 3).is_decreasing(strict=True)
                False

            ::

                >>> Sequence().is_decreasing(strict=True)
                True

        ..  container:: example

            With ``strict=False``:

            ::

                >>> Sequence(5, 4, 3, 2, 1, 0).is_decreasing(strict=False)
                True

            ::

                >>> Sequence(3, 3, 3, 2, 1, 0).is_decreasing(strict=False)
                True

            ::

                >>> Sequence(3, 3, 3, 3, 3, 3).is_decreasing(strict=False)
                True

            ::

                >>> Sequence().is_decreasing(strict=False)
                True

        Returns boolean.
        '''
        if strict:
            try:
                previous = None
                for current in self:
                    if previous is not None:
                        if not current < previous:
                            return False
                    previous = current
                return True
            except TypeError:
                return False
        else:
            try:
                previous = None
                for current in self:
                    if previous is not None:
                        if not current <= previous:
                            return False
                    previous = current
                return True
            except TypeError:
                return False

    def is_increasing(self, strict=True):
        r'''Is true when sequence increases.

        ..  container:: example

            With ``strict=True``:

            ::

                >>> Sequence(0, 1, 2, 3, 4, 5).is_increasing(strict=True)
                True

            ::

                >>> Sequence(0, 1, 2, 3, 3, 3).is_increasing(strict=True)
                False

            ::

                >>> Sequence(3, 3, 3, 3, 3, 3).is_increasing(strict=True)
                False

            ::

                >>> Sequence().is_increasing(strict=True)
                True

        ..  container:: example

            With ``strict=False``:

            ::

                >>> Sequence(0, 1, 2, 3, 4, 5).is_increasing(strict=False)
                True

            ::

                >>> Sequence(0, 1, 2, 3, 3, 3).is_increasing(strict=False)
                True

            ::

                >>> Sequence(3, 3, 3, 3, 3, 3).is_increasing(strict=False)
                True

            ::

                >>> Sequence().is_increasing(strict=False)
                True

        Returns boolean.
        '''
        if strict:
            try:
                previous = None
                for current in self:
                    if previous is not None:
                        if not previous < current:
                            return False
                    previous = current
                return True
            except TypeError:
                return False
        else:
            try:
                previous = None
                for current in self:
                    if previous is not None:
                        if not previous <= current:
                            return False
                    previous = current
                return True
            except TypeError:
                return False

    def is_permutation(self, length=None):
        '''Is true when sequence is a permutation.

        ..  container:: example

            Is true when sequence is a permutation:

            ::

                >>> Sequence(4, 5, 0, 3, 2, 1).is_permutation()
                True

        ..  container:: example

            Is false when sequence is not a permutation:

            ::

                >>> Sequence(1, 1, 5, 3, 2, 1).is_permutation()
                False

        Returns boolean.
        '''
        return tuple(sorted(self)) == tuple(range(len(self)))

    def is_repetition_free(self):
        '''Is true when sequence is repetition-free.

        ..  container:: example

            Is true when sequence contains no repetitions:

            ::

                >>> Sequence(0, 1, 2, 6, 7, 8).is_repetition_free()
                True

            Is true when sequence is empty:

            ::

                >>> Sequence().is_repetition_free()
                True

        ..  container:: example

            Is false when sequence contains repetitions:

            ::

                >>> Sequence(0, 1, 2, 2, 7, 8).is_repetition_free()
                False

        Returns boolean.
        '''
        from abjad.tools import sequencetools
        try:
            pairs = sequencetools.iterate_sequence_nwise(self)
            for left, right in pairs:
                if left == right:
                    return False
            return True
        except TypeError:
            return False

    def is_restricted_growth_function(self):
        '''Is true when sequence is a restricted growth function.

        ..  container:: example

            Is true when sequence is a restricted growth function:

            ::

                >>> Sequence(1, 1, 1, 1).is_restricted_growth_function()
                True

            ::


                >>> Sequence(1, 1, 1, 2).is_restricted_growth_function()
                True

            ::

                >>> Sequence(1, 1, 2, 1).is_restricted_growth_function()
                True

            ::

                >>> Sequence(1, 1, 2, 2).is_restricted_growth_function()
                True

        ..  container:: example

            Is false when sequence is not a restricted growth function:

            ::

                >>> Sequence(1, 1, 1, 3).is_restricted_growth_function()
                False

            ::

                >>> Sequence(17).is_restricted_growth_function()
                False

        A restricted growth function is a sequence ``l`` such that
        ``l[0] == 1`` and such that ``l[i] <= max(l[:i]) + 1`` for
        ``1 <= i <= len(l)``.

        Returns boolean.
        '''
        try:
            for i, n in enumerate(self):
                if i == 0:
                    if not n == 1:
                        return False
                else:
                    if not n <= max(self[:i]) + 1:
                        return False
            return True
        except TypeError:
            return False

    ### PUBLIC METHODS ###

    def reverse(self):
        '''Reverses this sequence.

        ::

            >>> sequencetools.Sequence(1, 2, 3, 4, 5).reverse()
            Sequence(5, 4, 3, 2, 1)

        Emits new sequence.
        '''
        return type(self)(*reversed(self))

    def rotate(self, n):
        '''Rotates this sequence.

        Rotates `sequence` to the right:

        ::

            >>> sequencetools.Sequence(*list(range(10))).rotate(4)
            Sequence(6, 7, 8, 9, 0, 1, 2, 3, 4, 5)

        Rotates `sequence` to the left:

        ::

            >>> sequencetools.Sequence(*list(range(10))).rotate(-3)
            Sequence(3, 4, 5, 6, 7, 8, 9, 0, 1, 2)

        Rotates `sequence` neither to the right nor the left:

        ::

            >>> sequencetools.Sequence(*list(range(10))).rotate(0)
            Sequence(0, 1, 2, 3, 4, 5, 6, 7, 8, 9)

        Emits new sequence.
        '''

        result = []
        if len(self):
            n = n % len(self)
            for element in self[-n:len(self)] + self[:-n]:
                result.append(element)
                #result.append(copy.deepcopy(element))
        return type(self)(*result)


collections.Sequence.register(Sequence)


# -*- encoding: utf-8 -*-
from abjad import *


def test_layouttools_set_line_breaks_by_line_duration_ge_01():
    r'''Iterate classes in expr and accumulate duration.
    Add line break after every total le line duration.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))
    staff.append(Measure((2, 8), "b'8 c''8"))
    layouttools.set_line_breaks_by_line_duration_ge(
        staff,
        Duration(4, 8),
        )

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
                \break
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
                \break
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_layouttools_set_line_breaks_by_line_duration_ge_02():
    r'''Iterate classes in expr and accumulate duration.
    Add line break after every total le line duration.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))
    staff.append(Measure((2, 8), "b'8 c''8"))
    layouttools.set_line_breaks_by_line_duration_ge(
        staff,
        Duration(1, 8),
        line_break_class=scoretools.Leaf,
        )

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                \break
                d'8
                \break
            }
            {
                e'8
                \break
                f'8
                \break
            }
            {
                g'8
                \break
                a'8
                \break
            }
            {
                b'8
                \break
                c''8
                \break
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-


def list_pitch_numbers_in_expr(expr):
    '''List pitch numbers in `expr`:

    ::

        >>> tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
        >>> pitchtools.list_pitch_numbers_in_expr(tuplet)
        (0, 2, 4)

    Returns tuple of zero or more numbers.
    '''
    from abjad.tools import pitchtools

    pitches = pitchtools.list_named_pitches_in_expr(expr)

    pitch_numbers = [pitch.pitch_number for pitch in pitches]
    pitch_numbers = tuple(pitch_numbers)

    return pitch_numbers


# -*- encoding: utf-8 -*-


def label_logical_ties_in_expr_with_logical_tie_durations(
    expr, markup_direction=Down):
    r'''Label logical ties in `expr` with both written logical tie duration
    and logical tie duration:

    ::

        >>> staff = Staff(r"\times 2/3 { c'8 ~ c'8 c'8 ~ } c'8")
        >>> labeltools.label_logical_ties_in_expr_with_logical_tie_durations(staff)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \times 2/3 {
                c'8 ~ _ \markup { \column { \small 1/4 \small 1/6 } }
                c'8
                c'8 ~ _ \markup { \column { \small 1/4 \small 5/24 } }
            }
            c'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    '''
    from abjad.tools import labeltools

    return labeltools.label_leaves_in_expr_with_leaf_durations(
        expr,
        label_durations=True,
        label_written_durations=True,
        markup_direction=markup_direction,
        )



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def color_measures_with_non_power_of_two_denominators_in_expr(
    expr,
    color='red',
    ):
    r'''Colors measures with non-power-of-two denominators in `expr`
    with `color`.

    ::

        >>> staff = Staff(2 * Measure((2, 8), "c'8 d'8"))
        >>> scoretools.scale_measure_denominator_and_adjust_measure_contents(staff[1], 3)
        Measure((3, 12), "c'8. d'8.", implicit_scaling=True)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8.
                    d'8.
                }
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> labeltools.color_measures_with_non_power_of_two_denominators_in_expr(staff, 'red')
        [Measure((3, 12), "c'8. d'8.", implicit_scaling=True)]

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \override Beam #'color = #red
                \override Dots #'color = #red
                \override NoteHead #'color = #red
                \override Staff.TimeSignature #'color = #red
                \override Stem #'color = #red
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8.
                    d'8.
                }
                \revert Beam #'color
                \revert Dots #'color
                \revert NoteHead #'color
                \revert Staff.TimeSignature #'color
                \revert Stem #'color
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns list of measures colored.

    Color names appear in LilyPond Learning Manual appendix B.5.
    '''
    from abjad.tools import labeltools

    # init measures colored
    measures_colored = []

    # color non-power-of-two measures in expr
    for measure in iterate(expr).by_class(scoretools.Measure):
        if measure.time_signature.has_non_power_of_two_denominator:
            labeltools.color_measure(measure, color)
            measures_colored.append(measure)

    # return measures colored
    return measures_colored



# -*- encoding: utf-8 -*-


def interlace_sequences(*sequences):
    '''Interlaces `sequences`.

    ::

        >>> k = range(100, 103)
        >>> l = range(200, 201)
        >>> m = range(300, 303)
        >>> n = range(400, 408)
        >>> sequencetools.interlace_sequences(k, l, m, n)
        [100, 200, 300, 400, 101, 301, 401, 102, 302, 402, 403, 404, 405, 406, 407]

    Returns list.
    '''
    from abjad.tools import sequencetools

    result = sequencetools.zip_sequences(sequences, truncate=False)
    result = sequencetools.flatten_sequence(result, depth=1)

    return result


# -*- encoding: utf-8 -*-
from abjad.tools import selectiontools


def make_repeated_notes_from_time_signatures(time_signatures, pitch="c'"):
    '''Make repated notes from `time_signatures`:

    ::

        scoretools.make_repeated_notes_from_time_signatures([(2, 8), (3, 32)])
        [Selection(Note("c'8"), Note("c'8")), Selection(Note("c'32"), Note("c'32"), Note("c'32"))]

    Make repeated notes with `pitch` from `time_signatures`:

    ::

        >>> scoretools.make_repeated_notes_from_time_signatures([(2, 8), (3, 32)], pitch="d''")
        [Selection(Note("d''8"), Note("d''8")), Selection(Note("d''32"), Note("d''32"), Note("d''32"))]

    Returns two-dimensional list of note lists.

    Use ``sequencetools.flatten_sequence()`` to flatten output if required.
    '''
    from abjad.tools import scoretools

    # init result
    result = []

    # iterate time signatures and make notes
    for time_signature in time_signatures:
        notes = scoretools.make_repeated_notes_from_time_signature(
            time_signature, pitch=pitch)
        result.append(notes)

    # return result
    return result


# -*- encoding: utf-8 -*-
import re
from abjad.tools.stringtools.is_dash_case \
	import hyphen_delimited_lowercase_regex_body


hyphen_delimited_lowercase_file_name_regex_body = """
    %s
    (\.[a-z,0-9]+)?
    """ % hyphen_delimited_lowercase_regex_body

hyphen_delimited_lowercase_file_name_regex = re.compile('^%s$' %
    hyphen_delimited_lowercase_file_name_regex_body, re.VERBOSE)

def is_dash_case_file_name(expr):
    r'''Is true when `expr` is a string and is hyphen-delimited lowercase 
    file name with extension.

    ..  container:: example

        ::

            >>> stringtools.is_dash_case_file_name('foo-bar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_dash_case_file_name('foo.bar.blah')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    if expr == '':
        return True

    return bool(hyphen_delimited_lowercase_file_name_regex.match(expr))


# -*- encoding: utf-8 -*-


# TODO: function should return nonreduced fraction generator instead
#       of pair generator.
def yield_nonreduced_fractions():
    '''Yields positive nonreduced fractions in Cantor diagonalized order:

    ::

        >>> generator = mathtools.yield_nonreduced_fractions()
        >>> for n in range(16):
        ...     next(generator)
        ...
        (1, 1)
        (2, 1)
        (1, 2)
        (1, 3)
        (2, 2)
        (3, 1)
        (4, 1)
        (3, 2)
        (2, 3)
        (1, 4)
        (1, 5)
        (2, 4)
        (3, 3)
        (4, 2)
        (5, 1)
        (6, 1)

    Returns generator.
    '''

    n = 2
    while True:
        if n % 2 == 0:
            lhs = 1
            while lhs < n:
                rhs = n - lhs
                yield lhs, rhs
                lhs += 1
        else:
            lhs = n - 1
            while 0 < lhs:
                rhs = n - lhs
                yield lhs, rhs
                lhs -= 1
        n += 1



# -*- encoding: utf-8 -*-
import re


space_delimited_lowercase_regex = re.compile(
    '^(([a-z,0-9]+[ ]+)*[a-z,0-9]+)?$')

def is_space_delimited_lowercase(expr):
    r'''Is true when `expr` is a string and is space-delimited lowercase.

    ..  container:: example

        ::

            >>> stringtools.is_space_delimited_lowercase('foo bar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_space_delimited_lowercase('foo_bar')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(space_delimited_lowercase_regex.match(expr))


# -*- encoding: utf-8 -*-
import collections
from abjad.tools.abctools import AbjadObject


class StringNumber(AbjadObject):
    r'''String number indicator.

    ::

        >>> indicator = indicatortools.StringNumber(1)
        >>> print(format(indicator))
        indicatortools.StringNumber(
            numbers=(1,),
            )

    ::

        >>> indicator = indicatortools.StringNumber((2, 3))
        >>> print(format(indicator))
        indicatortools.StringNumber(
            numbers=(2, 3),
            )

    '''

    ### CLASS VARIABLES

    __slots__ = (
        '_numbers',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        numbers=None,
        ):
        numbers = numbers or ()
        if isinstance(numbers, type(self)):
            numbers = numbers.numbers
        elif not isinstance(numbers, collections.Sequence):
            numbers = (numbers,)
        numbers = tuple(int(x) for x in numbers)
        assert all(0 < x < 7 for x in numbers)
        self._numbers = tuple(numbers)

    ### PUBLIC PROPERTIES ###

    @property
    def numbers(self):
        r'''Gets string numbers.

        ::

            >>> indicator = indicatortools.StringNumber((2, 3))
            >>> indicator.numbers
            (2, 3)

        '''
        return self._numbers

    @property
    def roman_numerals(self):
        r'''Gets string numbers as roman numerals.

            >>> indicator = indicatortools.StringNumber((3, 4))
            >>> indicator.roman_numerals
            ('iii', 'iv')

        '''
        numerals = ('i', 'ii', 'iii', 'iv', 'v', 'vi')
        result = []
        for x in self.numbers:
            numeral = numerals[x - 1]
            result.append(numeral)
        return tuple(result)



# -*- encoding: utf-8 -*-
import inspect
import os


class TestManager(object):
    r'''Manages test logic.
    '''

    ### PRIVATE METHODS ###

    @staticmethod
    def _compare_backup(path):
        if isinstance(path, str):
            paths = [path]
        elif (isinstance(path, (tuple, list))):
            paths = path
        else:
            raise TypeError(path)
        for path in paths:
            backup_path = path + '.backup'
            if not TestManager.compare_files(path, backup_path):
                return False
        return True

    @staticmethod
    def _compare_lys(path_1, path_2):
        r'''Compares LilyPond file `path_1` to LilyPond file `path_2`.

        Performs line-by-line comparison.

        Discards blank lines.

        Discards any LilyPond version statements.

        Discards any lines beginning with ``%``.

        Returns boolean.
        '''
        file_1_lines = TestManager._normalize_ly(path_1)
        file_2_lines = TestManager._normalize_ly(path_2)
        return file_1_lines == file_2_lines

    @staticmethod
    def _compare_pdfs(path_1, path_2):
        r'''Compares PDF `path_1` to PDF `path_2`.

        Performs line-by-line comparison.

        Discards blank lines.

        Discards lines that contain any of the following strings:

        * ``/ID``
        * ``/CreationDate``
        * ``/ModDate``
        * ``xmp:CreateDate``
        * ``xmp:ModifyDate``
        * ``xapMM:DocumentID``
        * ``rdf:about``

        Discards first (binary) stream object in PDF;
        possibly first stream contains binary-encoded creator
        or timestamp information that can vary from one creation
        of a PDF to another.

        Returns boolean.
        '''
        file_1_lines = TestManager._normalize_pdf(path_1)
        file_2_lines = TestManager._normalize_pdf(path_2)
        return file_1_lines == file_2_lines

    @staticmethod
    def _compare_text_files(path_1, path_2):
        r'''Compares text file `path_1` to text file `path_2`.

        Performs line-by-line comparison.

        Discards blank lines.

        Trims whitespace from the end of each line.

        Returns boolean.
        '''
        file_1_lines, file_2_lines = [], []
        with open(path_1, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                file_1_lines.append(line)
        with open(path_2, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                file_2_lines.append(line)
        return file_1_lines == file_2_lines

    @staticmethod
    def _normalize_ly(path):
        lines = []
        with open(path, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                if line.startswith(r'\version'):
                    continue
                elif line.startswith('%'):
                    continue
                lines.append(line)
        return lines

    @staticmethod
    def _normalize_pdf(path):
        lines = []
        with open(path, 'rb') as file_pointer:
            contents = file_pointer.read()
            for line in contents.splitlines():
                line = line.strip()
                if line == b'':
                    continue
                elif line.startswith(b'%'):
                    continue
                elif b'/ID' in line:
                    continue
                elif b'/Creator' in line:
                    continue
                elif b'/CreationDate' in line:
                    continue
                elif b'/ModDate' in line:
                    continue
                elif b'xmp:CreateDate' in line:
                    continue
                elif b'xmp:CreatorTool' in line:
                    continue
                elif b'xmp:ModifyDate' in line:
                    continue
                elif b'xapMM:DocumentID' in line:
                    continue
                elif b'rdf:about' in line:
                    continue
                lines.append(line)
        # discard first stream in document: can contain differences;
        # no idea why; possibly first stream includes binary-encoded
        # creator information or binary-encoded timestamp information
        new_lines = []
        found_first_stream = False
        in_first_stream = False
        for line in lines:
            if line == b'stream' and not found_first_stream:
                found_first_stream = True
                in_first_stream = True
            elif line.endswith(b'endstream') and in_first_stream:
                in_first_stream = False
                continue
            if in_first_stream:
                continue
            else:
                new_lines.append(line)
        return new_lines

    ### PUBLIC METHODS ###

    @staticmethod
    def apply_additional_layout(lilypond_file):
        r'''Configures multiple-voice rhythmic staves in `lilypond_file`.

        Operates in place.

        Returns none.
        '''
        from abjad.tools import layouttools
        from abjad.tools import lilypondfiletools
        from abjad.tools import scoretools
        from abjad.tools import topleveltools
        # configure multiple-voice rhythmic staves
        for staff in topleveltools.iterate(lilypond_file.score_block.items[0]
            ).by_class(scoretools.Staff):
            if staff.is_simultaneous:
                assert len(staff) == 2
                voice_1 = staff[0]
                topleveltools.override(voice_1).note_head.Y_offset = 0.5
                topleveltools.override(voice_1).stem.direction = Up
                voice_2 = staff[1]
                topleveltools.override(voice_2).note_head.Y_offset = -0.5
                topleveltools.override(voice_2).stem.direction = Down
                spacing_vector = layouttools.make_spacing_vector(0, 0, 6, 0)
                manager = topleveltools.override(staff)
                manager.vertical_axis_group.staff_staff_spacing = spacing_vector
        # provide more space between staves with pitched notes
        for staff in topleveltools.iterate(
            lilypond_file.score_block.items[0]).by_class(scoretools.Staff):
            if not (isinstance(staff, scoretools.Staff) and
                staff.context_name == 'RhythmicStaff'):
                for item in lilypond_file.layout_block.items:
                    if isinstance(item, lilypondfiletools.ContextBlock):
                        if item.source_context_name == 'StaffGroup':
                            break
                else:
                    message = 'no staff group context block found.'
                    raise Exception(message)
                spacing_vector = layouttools.make_spacing_vector(0, 0, 6, 0)
                manager = topleveltools.override(item)
                manager.vertical_axis_group.staff_staff_spacing = spacing_vector
            break

    @staticmethod
    def compare(string_1, string_2):
        r'''Compares `string_1` to `string_2`.

        Massage newlines.

        Returns boolean.
        '''
        if not isinstance(string_1, str):
            string_1 = format(string_1)
        split_lines = string_2.split('\n')
        if not split_lines[0] or split_lines[0].isspace():
            split_lines.pop(0)
        if not split_lines[-1] or split_lines[-1].isspace():
            split_lines.pop(-1)
        for indent_width, character in enumerate(split_lines[0]):
            if character != ' ':
                break
        tab_string = 4 * ' '
        massaged_lines = []
        for split_line in split_lines:
            massaged_line = split_line[indent_width:]
            massaged_lines.append(massaged_line)
        massaged_string = '\n'.join(massaged_lines)
        return string_1.replace('\t', '    ') == massaged_string

    @staticmethod
    def compare_files(path_1, path_2):
        r'''Compares file `path_1` to file `path_2`.

        For all file types::

        * Performs line-by-line comparison
        * Discards blank lines

        For LilyPond files, additionally::

        * Discards any LilyPond version statements
        * Discards any lines beginning with ``%``

        For PDFs, additionally discards lines that contain any of the 
        following strings:

        * ``/ID``
        * ``/CreationDate``
        * ``/ModDate``
        * ``xmp:CreateDate``
        * ``xmp:ModifyDate``
        * ``xapMM:DocumentID``
        * ``rdf:about``

        Discards first (binary) stream object in PDF;
        possibly first stream contains binary-encoded creator
        or timestamp information that can vary from one creation
        of a PDF to another.

        Returns true when files compare the same and false when files compare
        differently.
        '''
        if os.path.exists(path_1) and not os.path.exists(path_2):
            return False
        elif not os.path.exists(path_1) and os.path.exists(path_2):
            return False
        elif not os.path.exists(path_1) and not os.path.exists(path_2):
            return True
        if path_1.endswith('.backup'):
            path_1 = path_1.strip('.backup')
        if path_2.endswith('.backup'):
            path_2 = path_2.strip('.backup')
        base_1, extension_1 = os.path.splitext(path_1)
        base_2, extension_2 = os.path.splitext(path_2)
        assert extension_1 == extension_2
        if extension_1 == '.ly':
            return TestManager._compare_lys(path_1, path_2)
        elif extension_1 == '.pdf':
            return TestManager._compare_pdfs(path_1, path_2)
        else:
            return TestManager._compare_text_files(path_1, path_2)

    @staticmethod
    def get_current_function_name():
        r'''Gets current function name.

        ::

            >>> def foo():
            ...        function_name = systemtools.TestManager.get_current_function_name()
            ...        print('Function name is {!r}.'.format(function_name))

        ::

            >>> foo()
            Function name is 'foo'.

        Call this function within the implementation of any ofther function.

        Returns enclosing function name as a string or else none.
        '''
        stack = inspect.stack()
        # per the inspect module doc page ...
        try:
            parent_frame_record = stack[1]
            parent_frame = parent_frame_record[0]
            parent_frame_info = inspect.getframeinfo(parent_frame)
            parent_frame_function_name = parent_frame_info.function
            return parent_frame_function_name
        # ... destroy frame to avoid reference cycle
        finally:
            del stack

    @staticmethod
    def read_test_output(full_file_name, current_function_name):
        r'''Reads test output.

        Returns list.
        '''
        segment_ly_file_name = '{}.ly'.format(current_function_name)
        directory_name = os.path.dirname(full_file_name)
        segment_ly_path_name = os.path.join(
            directory_name,
            segment_ly_file_name,
            )
        with open(segment_ly_path_name, 'r') as f:
            string = f.read()
        return string

    @staticmethod
    def test_function_name_to_title_lines(test_function_name):
        r'''Changes `test_function_name` to title lines.

        Returns list.
        '''
        from abjad.tools import sequencetools
        title_lines = []
        test_function_name = test_function_name[5:]
        if '__' in test_function_name:
            left_half, right_half = test_function_name.split('__')
            left_half = left_half.replace('_', ' ')
            title_lines.append(left_half)
            parts = right_half.split('_')
        else:
            parts = test_function_name.split('_')
        test_number = int(parts[-1])
        parts.pop(-1)
        if parts[0][0].isupper() and 1 < len(parts[0]):
            title_lines.append(parts.pop(0))
        lengths = [len(part) for part in parts]
        if 35 < sum(lengths):
            halves = sequencetools.partition_sequence_by_ratio_of_weights(
                lengths,
                [1, 1],
                )
            left_count = len(halves[0])
            right_count = len(halves[-1])
            assert left_count + right_count == len(lengths)
            left_parts = parts[:left_count]
            title_lines.append(' '.join(left_parts))
            right_parts = parts[-right_count:]
            right_parts.append(str(test_number))
            title_lines.append(' '.join(right_parts))
        else:
            title_words = ' '.join(parts)
            if 'schematic example' in title_words:
                space = ''
            else:
                space = ' '
            title = '{}{}{}'.format(title_words, space, test_number)
            title_lines.append(title)
        return title_lines

    @staticmethod
    def write_test_output(
        output,
        full_file_name,
        test_function_name,
        cache_ly=False,
        cache_pdf=False,
        go=False,
        render_pdf=False,
        ):
        r'''Writes test output.

        Returns none.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import schemetools
        from abjad.tools import scoretools
        from abjad.tools import systemtools
        from abjad.tools import topleveltools
        if go:
            cache_ly = cache_pdf = render_pdf = True
        if not any([cache_ly, cache_pdf, render_pdf]):
            return
        if isinstance(output, scoretools.Score):
            lilypond_file = \
                lilypondfiletools.make_floating_time_signature_lilypond_file(
                    output)
            TestManager.apply_additional_layout(lilypond_file)
            score = output
        elif isinstance(output, lilypondfiletools.LilyPondFile):
            lilypond_file = output
            score = lilypond_file.score_block[0]
        else:
            message = 'output must be score or LilyPond file: {!r}.'
            message = message.format(output)
            raise TypeError(message)
        title_lines = TestManager.test_function_name_to_title_lines(
            test_function_name)
        lilypond_file.header_block.title = \
            markuptools.make_centered_title_markup(
                title_lines,
                font_size=6,
                vspace_before=2,
                vspace_after=4,
                )
        parent_directory_name = os.path.dirname(full_file_name)
        if render_pdf:
            topleveltools.show(lilypond_file)
        if cache_pdf:
            file_name = '{}.pdf'.format(test_function_name)
            pdf_path_name = os.path.join(parent_directory_name, file_name)
            topleveltools.persist(lilypond_file).as_pdf(pdf_path_name)
        if cache_ly:
            file_name = '{}.ly'.format(test_function_name)
            ly_path_name = os.path.join(parent_directory_name, file_name)
            with open(ly_path_name, 'w') as f:
                f.write(format(score))


# -*- encoding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class Beam(Spanner):
    r'''A beam.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8 g'2")
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8
                d'8
                e'8
                f'8
                g'2
            }

        ::

            >>> beam = Beam()
            >>> attach(beam, staff[:2])
            >>> beam = Beam()
            >>> attach(beam, staff[2:4])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8 [
                d'8 ]
                e'8 [
                f'8 ]
                g'2
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        direction = stringtools.arg_to_tridirectional_lilypond_symbol(
            direction)
        self._direction = direction

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        Spanner._copy_keyword_args(self, new)
        new._direction = self.direction

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
            if self.direction is not None:
                string = '{} ['.format(self.direction)
            else:
                string = '['
            lilypond_format_bundle.right.spanner_starts.append(string)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
            string = ']'
            if self._is_my_first_leaf(leaf):
                lilypond_format_bundle.right.spanner_starts.append(string)
            else:
                lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC METHODS ###

    @staticmethod
    def _is_beamable_component(expr):
        '''Is true when `expr` is a beamable component. Otherwise false.

        ::

            >>> staff = Staff(r"r32 a'32 ( [ gs'32 fs''32 \staccato f''8 ) ]")
            >>> staff.extend(r"r8 e''8 ( ef'2 )")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> for leaf in staff.select_leaves():
            ...     beam = spannertools.Beam
            ...     result = beam._is_beamable_component(leaf)
            ...     print('{:<8}\t{}'.format(leaf, result))
            ... 
            r32     False
            a'32    True
            gs'32   True
            fs''32  True
            f''8    True
            r8      False
            e''8    True
            ef'2    False

        Returns boolean.
        '''
        from abjad.tools import scoretools
        if isinstance(expr, (scoretools.Note, scoretools.Chord)):
            if 0 < expr.written_duration.flag_count:
                return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets direction of beam.

        Returns up or down.
        '''
        return self._direction



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondFile(AbjadObject):
    r'''A LilyPond file.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> lilypond_file = lilypondfiletools.make_basic_lilypond_file(staff)
            >>> comment = 'File construct as an example.'
            >>> lilypond_file.file_initial_user_comments.append(comment)
            >>> comment = 'Parts shown here for positioning.'
            >>> lilypond_file.file_initial_user_comments.append(comment)
            >>> file_name = 'external-settings-file-1.ly'
            >>> lilypond_file.file_initial_user_includes.append(file_name)
            >>> file_name = 'external-settings-file-2.ly'
            >>> lilypond_file.file_initial_user_includes.append(file_name)
            >>> lilypond_file.default_paper_size = 'a5', 'portrait'
            >>> lilypond_file.global_staff_size = 16
            >>> lilypond_file.header_block.composer = Markup('Josquin')
            >>> lilypond_file.header_block.title = Markup('Missa sexti tonus')
            >>> lilypond_file.layout_block.indent = 0
            >>> lilypond_file.layout_block.left_margin = 15
            >>> lilypond_file
            <LilyPondFile(4)>

        ::

            >>> print(format(lilypond_file)) # doctest: +SKIP
            % 2004-01-14 17:29

            % File construct as an example.
            % Parts shown here for positioning.

            \version "2.19.0"
            \include "english.ly"

            \include "external-settings-file-1.ly"
            \include "external-settings-file-2.ly"

            #(set-default-paper-size "a5" 'portrait)
            #(set-global-staff-size 16)

            \header {
                composer = \markup { Josquin }
                title = \markup { Missa sexti tonus }
            }

            \layout {
                indent = #0
                left-margin = #15
            }

            \paper {
            }

            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }

        ::

            >>> show(lilypond_file) # doctest: +SKIP

    '''

    def __init__(self):
        from abjad.tools import lilypondfiletools
        self._items = []
        self._file_initial_system_comments = []
        token = lilypondfiletools.DateTimeToken()
        self._file_initial_system_comments.append(token)
        self._file_initial_user_comments = []
        self._file_initial_system_includes = []
        token = lilypondfiletools.LilyPondVersionToken()
        self._file_initial_system_includes.append(token)
        token = lilypondfiletools.LilyPondLanguageToken()
        self._file_initial_system_includes.append(token)
        self._file_initial_user_includes = []
        self.default_paper_size = None
        self.global_staff_size = None
        self.use_relative_includes = False

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond file.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getitem__(self, name):
        r'''Gets LilyPond file item with `name`.

        ..  container:: example

            ::

                >>> lilypond_file['header']
                <Block(name='header')>

        Raises key error when no item with `name` is found.
        '''
        for item in self.items:
            if getattr(item, 'name', None) == name:
                return item
        raise KeyError

    def __illustrate__(self):
        r'''Illustrates LilyPond file.

        Returns LilyPond file unchanged.
        '''
        return self

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond file.

        ..  container:: example

            ::

                >>> lilypond_file
                <LilyPondFile(4)>

        Returns string.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.get_repr_format(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        result = []
        result.extend(self._formatted_file_initial_system_comments)
        result.extend(self._formatted_file_initial_user_comments)
        result.extend(self._formatted_file_initial_system_includes)
        if self.use_relative_includes:
            string = "#(ly:set-option 'relative-includes #t)"
            result.append(string)
        result.extend(self._formatted_file_initial_user_includes)
        result.extend(self._formatted_file_initial_scheme_settings)
        result.extend(self._formatted_blocks)
        return result

    @property
    def _formatted_blocks(self):
        result = []
        for x in self.items:
            if '_lilypond_format' in dir(x) and not isinstance(x, str):
                lilypond_format = format(x)
                if lilypond_format:
                    result.append(lilypond_format)
            else:
                result.append(str(x))
        return result

    @property
    def _formatted_file_initial_scheme_settings(self):
        result = []
        default_paper_size = self.default_paper_size
        if default_paper_size is not None:
            dimension, orientation = default_paper_size
            string = "#(set-default-paper-size \"{}\" '{})"
            string = string.format(dimension, orientation)
            result.append(string)
        global_staff_size = self.global_staff_size
        if global_staff_size is not None:
            string = '#(set-global-staff-size {})'
            string = string.format(global_staff_size)
            result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_system_comments(self):
        result = []
        for comment in self.file_initial_system_comments:
            if '_lilypond_format' in dir(comment) and \
                not isinstance(comment, str):
                lilypond_format = format(comment)
                if lilypond_format:
                    string = '% {}'.format(comment)
                    result.append(string)
            else:
                string = '% {!s}'.format(comment)
                result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_system_includes(self):
        result = []
        for file_initial_include in self.file_initial_system_includes:
            if isinstance(file_initial_include, str):
                string = r'\include "{}"'.format(file_initial_include)
                result.append(string)
            else:
                result.append(format(file_initial_include))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_user_comments(self):
        result = []
        for comment in self.file_initial_user_comments:
            if '_lilypond_format' in dir(comment) and \
                not isinstance(comment, str):
                lilypond_format = format(comment)
                if lilypond_format:
                    string = '% {}'.format(comment)
                    result.append(string)
            else:
                string = '% {!s}'.format(comment)
                result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_file_initial_user_includes(self):
        result = []
        for file_initial_include in self.file_initial_user_includes:
            if isinstance(file_initial_include, str):
                string = r'\include "{}"'.format(file_initial_include)
                result.append(string)
            else:
                result.append(format(file_initial_include))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _lilypond_format(self):
        return '\n\n'.join(self._format_pieces)

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = []
        if self.items:
            positional_argument_values.append(len(self.items))
        positional_argument_values = tuple(positional_argument_values)
        return systemtools.StorageFormatSpecification(
            self,
            is_bracketed=True,
            is_indented=False,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def default_paper_size(self):
        r'''Gets default paper size of LilyPond file.

        ..  container:: example

            ::

                >>> lilypond_file.default_paper_size
                ('a5', 'portrait')

        Returns pair or none.
        '''
        return self._default_paper_size

    @default_paper_size.setter
    def default_paper_size(self, args):
        assert args is None or len(args) == 2
        self._default_paper_size = args

    @property
    def file_initial_system_comments(self):
        r'''Gets file-initial system comments of LilyPond file.

        ..  container:: example

            ::

                >>> for x in lilypond_file.file_initial_system_comments: # doctest: +SKIP
                ...     x
                DateTimeToken('2014-01-04 17:16')

        Returns list.
        '''
        return self._file_initial_system_comments

    @property
    def file_initial_system_includes(self):
        r'''Gets file-initial system include commands of LilyPond file.

        ..  container:: example

            ::

                >>> for x in lilypond_file.file_initial_system_includes:
                ...     x
                LilyPondVersionToken('2.19.11')
                LilyPondLanguageToken('english')

        Returns list.
        '''
        return self._file_initial_system_includes

    @property
    def file_initial_user_comments(self):
        r'''Gets file-initial user comments of Lilypond file.

        ..  container:: example

            ::

                >>> for x in lilypond_file.file_initial_user_comments:
                ...     x
                'File construct as an example.'
                'Parts shown here for positioning.'

        Returns list.
        '''
        return self._file_initial_user_comments

    @property
    def file_initial_user_includes(self):
        r'''Gets file-initial user include commands of LilyPond file.

        ..  container:: example

            ::

                >>> for x in lilypond_file.file_initial_user_includes:
                ...     x
                'external-settings-file-1.ly'
                'external-settings-file-2.ly'

        Returns list.
        '''
        return self._file_initial_user_includes

    @property
    def global_staff_size(self):
        r'''Gets global staff size of LilyPond file.

        ..  container:: example

            ::

                >>> lilypond_file.global_staff_size
                16

        Returns number.
        '''
        return self._global_staff_size

    @global_staff_size.setter
    def global_staff_size(self, arg):
        assert isinstance(arg, (int, float, int, type(None)))
        self._global_staff_size = arg

    @property
    def items(self):
        r'''Gets items in LilyPond file.

        ..  container:: example

            ::

                >>> for item in lilypond_file.items:
                ...     item
                <Block(name='header')>
                <Block(name='layout')>
                <Block(name='paper')>
                <Block(name='score')>

        Returns list.
        '''
        return self._items

    @property
    def use_relative_includes(self):
        r'''Gets boolean flag to use relative include paths.

        ..  container:: example

            ::

                >>> lilypond_file.use_relative_includes
                False

        Returns boolean.
        '''
        return self._use_relative_includes

    @use_relative_includes.setter
    def use_relative_includes(self, arg):
        self._use_relative_includes = bool(arg)


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_Ratio___init___01():
    r'''Initializes ratio from empty input.
    '''

    ratio = mathtools.Ratio()

    assert ratio == mathtools.Ratio(1, 1)


def test_mathtools_Ratio___init___02():
    r'''Initialize from integers.
    '''

    ratio = mathtools.Ratio(1, 2, 1)

    assert isinstance(ratio, mathtools.Ratio)


def test_mathtools_Ratio___init___03():
    r'''Initialize from a tuple or list.
    '''

    ratio = mathtools.Ratio((1, 2, 1))

    assert isinstance(ratio, mathtools.Ratio)


def test_mathtools_Ratio___init___04():

    assert pytest.raises(Exception, 'mathtools.Ratio(1, 2, 0)')


# -*- encoding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools import stringtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Markup(AbjadObject):
    r'''A LilyPond markup.

    ..  container:: example

        Initializes from string:

        ::

            >>> string = r'\bold { "This is markup text." }'
            >>> markup = Markup(string)
            >>> show(markup) # doctest: +SKIP

        ..  doctest::

            >>> print(format(markup))
            \markup { \bold { "This is markup text." } }

    ..  container:: example

        Initializes from other markup:

        ::

            >>> markup_1 = Markup('foo', direction=Up)
            >>> markup_2 = Markup(markup_1, direction=Down)
            >>> show(markup_2) # doctest: +SKIP

        ..  doctest::

            >>> print(format(markup_1))
            ^ \markup { foo }

        ..  doctest::

            >>> print(format(markup_2))
            _ \markup { foo }

    ..  container:: example

        Attaches markup to score components:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> string = r'\italic { "This is also markup text." }'
            >>> markup = Markup(string, direction=Up)
            >>> attach(markup, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8
                    ^ \markup {
                        \italic
                            {
                                "This is also markup text."
                            }
                        }
                d'8
                e'8
                f'8
            }

    Set `direction` to ``Up``, ``Down``, ``'neutral'``,
    ``'^'``, ``'_'``, ``'-'`` or None.

    Markup objects are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contents',
        '_direction',
        '_format_slot',
        )

    ### INITIALIZER ###

    def __init__(self, contents=None, direction=None):
        from abjad.tools import lilypondparsertools
        from abjad.tools import markuptools
        if contents is None:
            new_contents = ('',)
        elif isinstance(contents, str):
            to_parse = r'\markup {{ {} }}'.format(contents)
            parsed = lilypondparsertools.LilyPondParser()(to_parse)
            if all(isinstance(x, str) for x in parsed.contents):
                new_contents = (' '.join(parsed.contents),)
            else:
                new_contents = tuple(parsed.contents)
        elif isinstance(contents, markuptools.MarkupCommand):
            new_contents = (contents,)
        elif isinstance(contents, type(self)):
            direction = direction or contents._direction
            new_contents = tuple(contents._contents)
        elif isinstance(contents, (list, tuple)) and 0 < len(contents):
            new_contents = []
            for arg in contents:
                if isinstance(arg, (str, markuptools.MarkupCommand)):
                    new_contents.append(arg)
                else:
                    new_contents.append(str(arg))
            new_contents = tuple(new_contents)
        else:
            new_contents = (str(contents),)
        self._contents = new_contents
        self._format_slot = 'right'
        direction = \
            stringtools.arg_to_tridirectional_ordinal_constant(direction)
        self._direction = direction

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies markup.

        ..  container:: example

            ::

                >>> import copy
                >>> string = r'\bold { allegro ma non troppo }'
                >>> markup = Markup(string)
                >>> new_markup = copy.copy(markup)
                >>> show(new_markup) # doctest: +SKIP

        Returns new markup.
        '''
        return type(self)(
            self._contents,
            direction=self._direction,
            )

    def __eq__(self, expr):
        r'''Is true when `expr` is a markup with format equal to that
        of this markup. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if format(self) == format(expr):
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats markup.

        ..  container:: example

            ::

                >>> string = r'\bold { allegro ma non troppo }'
                >>> markup = Markup(string)
                >>> print(format(markup))
                \markup {
                    \bold
                        {
                            allegro
                            ma
                            non
                            troppo
                        }
                    }

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'lilypond'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes markup.

        Returns integer.
        '''
        return hash((type(self).__name__, self.contents))

    # TODO: add verifiable code in doctest
    def __illustrate__(self):
        r'''Illustrates markup.

        ..  container:: example

            ::

                >>> string = r'\bold { allegro ma non troppo }'
                >>> markup = Markup(string)
                >>> show(markup) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        lilypond_file = lilypondfiletools.make_basic_lilypond_file()
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        lilypond_file.items.append(self)
        return lilypond_file

    def __str__(self):
        r'''Gets string representation of markup.

        ..  container:: example

            ::

                >>> string = r'\bold { allegro ma non troppo }'
                >>> markup = Markup(string)
                >>> print(str(markup))
                \markup {
                    \bold
                        {
                            allegro
                            ma
                            non
                            troppo
                        }
                    }

        Returns string.
        '''
        return self._lilypond_format

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='contents',
                display_string='arg',
                command='ag',
                editor=idetools.getters.get_string,
                ),
            systemtools.AttributeDetail(
                name='direction',
                command='dr',
                editor=idetools.getters.get_direction_string,
                ),
            )

    @property
    def _format_pieces(self):
        return self._get_format_pieces()

    @property
    def _lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        indent = '\t'
        direction = ''
        if self.direction is not None:
            direction = stringtools.arg_to_tridirectional_lilypond_symbol(
                self.direction)
        # none
        if self.contents is None:
            return [r'\markup {}']
        # a single string
        if len(self.contents) == 1 and isinstance(self.contents[0], str):
            content = self.contents[0]
            if '"' in content:
                content = schemetools.Scheme.format_scheme_value(content)
            if content:
                content = '{{ {} }}'.format(content)
            else:
                content = '{}'
            if direction:
                return [r'{} \markup {}'.format(direction, content)]
            return [r'\markup {}'.format(content)]
        # multiple strings or markup commands
        if direction:
            pieces = [r'{} \markup {{'.format(direction)]
        else:
            pieces = [r'\markup {']
        for content in self.contents:
            if isinstance(content, str):
                content = schemetools.Scheme.format_scheme_value(content)
                pieces.append('{}{}'.format(indent, content))
            else:
                pieces.extend(['{}{}'.format(indent, x) for x in
                    content._get_format_pieces()])
        pieces.append('{}}}'.format(indent))
        return pieces

    ### PUBLIC PROPERTIES ###

    @property
    def contents(self):
        r'''Gets contents of markup.

        ..  container:: example

            ::

                >>> string = r'\bold { "This is markup text." }'
                >>> markup = Markup(string)
                >>> show(markup) # doctest: +SKIP

            ::

                >>> markup.contents
                (MarkupCommand('bold', ['This is markup text.']),)

        Returns tuple.
        '''
        return self._contents

    @property
    def direction(self):
        r'''Gets direction of markup.

        ..  container:: example

            ::

                >>> string = r'\bold { "This is markup text." }'
                >>> markup = Markup(string, direction=Up)
                >>> show(markup) # doctest: +SKIP

            ::

                >>> markup.direction
                Up

        Returns up, down, center or none.
        '''
        return self._direction


# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_zip_sequences_01():
    r'''Can take two lists of the same size.
    '''

    sequences = ([1, 2], ['a', 'b'])
    sequence_2 = sequencetools.zip_sequences(sequences, cyclic=True)
    assert sequence_2 == [(1, 'a'), (2, 'b')]


def test_sequencetools_zip_sequences_02():
    r'''Can take two lists of the different sizes.
    The list with the shortest size is cycled through.
    '''

    sequences = ([1, 2, 3], ['a', 'b'])
    result = sequencetools.zip_sequences(sequences, cyclic=True)
    assert result == [(1, 'a'), (2, 'b'), (3, 'a')]

    sequences = ([1, 2], ['a', 'b', 'c'])
    result = sequencetools.zip_sequences(sequences, cyclic=True)
    assert result == [(1, 'a'), (2, 'b'), (1, 'c')]


def test_sequencetools_zip_sequences_03():
    r'''Handles more than two iterables.
    '''

    a = [10, 11, 12]
    b = [20, 21]
    c = [30, 31, 32, 33]
    sequences = (a, b, c)
    result = sequencetools.zip_sequences(sequences, cyclic=True)

    assert result == [(10, 20, 30), (11, 21, 31), (12, 20, 32), (10, 21, 33)]


def test_sequencetools_zip_sequences_04():
    r'''Zip and do not truncate to the length of the shortest list.
    '''

    sequences = ([1, 2, 3, 4], [11, 12, 13])
    result = sequencetools.zip_sequences(sequences, truncate=False)
    assert result == [(1, 11), (2, 12), (3, 13), (4,)]


def test_sequencetools_zip_sequences_05():
    r'''Zip and do not truncate to the length of the shortest list.
    '''

    sequences = ([1, 2, 3], [11, 12, 13, 14])
    result = sequencetools.zip_sequences(sequences, truncate=False)
    assert result == [(1, 11), (2, 12), (3, 13), (14,)]


# -*- encoding: utf-8 -*-
import numbers


def integer_equivalent_number_to_integer(number):
    '''Integer-equivalent `number` to integer.

    ::

        >>> mathtools.integer_equivalent_number_to_integer(17.0)
        17

    Returns noninteger-equivalent number unchanged:

    ::

        >>> mathtools.integer_equivalent_number_to_integer(17.5)
        17.5

    Raises type error on nonnumber input.

    Returns number.
    '''
    from abjad.tools import mathtools

    if not isinstance(number, numbers.Number):
        message = 'must be number: {!r}.'
        message = message.format(number)
        raise TypeError(message)

    if mathtools.is_integer_equivalent_number(number):
        return int(number)
    else:
        return number


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.indicatortools import Tempo
from abjad.tools.layouttools import SpacingIndication


def test_layouttools_SpacingIndication___repr___01():
    r'''Repr is evaluable.
    '''

    indication_1 = SpacingIndication(Tempo(Duration(1, 8), 44), Duration(1, 68))
    indication_2 = SpacingIndication(indication_1)

    assert indication_1 is not indication_2
    assert indication_1 == indication_2


# -*- encoding: utf-8 -*-
import argparse
import os
from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class AbjDevScript(DeveloperScript):
    '''`AbjDevScript` is the commandline entry-point to the Abjad
    developer scripts catalog.

    Can be accessed on the commandline via `abj-dev` or `ajv`:

    ..  shell::

        ajv --help

    `ajv` supports subcommands similar to `svn`:

    ..  shell::

        ajv api --help

    '''

    ### SPECIAL METHODS ###

    def __call__(self, args=None):
        r'''Calls script.
        '''
        if args is None:
            args = self.argument_parser.parse_known_args()
        else:
            if isinstance(args, str):
                args = args.split()
            elif not isinstance(args, (list, tuple)):
                message = 'must be str, list, tuple or none: {!r}.'
                message = message.format(args)
                raise ValueError(message)
            args = self.argument_parser.parse_known_args(args)
        self.process_args(args)

    ### PUBLIC PROPERTIES ###

    @property
    def developer_script_aliases(self):
        r'''Developer script aliases.
        '''
        scripting_groups = []
        aliases = {}
        for developer_script_class in self.developer_script_classes:
            instance = developer_script_class()

            if instance.alias is not None:

                if instance.scripting_group is not None:
                    scripting_groups.append(instance.scripting_group)
                    entry = (instance.scripting_group, instance.alias)
                    if (instance.scripting_group,) in aliases:
                        message = 'alias conflict between scripting group'
                        message += ' {!r} and {}'
                        message = message.format(
                            instance.scripting_group,
                            aliases[(instance.scripting_group,)].__name__,
                            )
                        raise Exception(message)
                    if entry in aliases:
                        message = 'alias conflict between {} and {}'
                        message = message.format(
                            aliases[entry].__name__,
                            developer_script_class.__name__,
                            )
                        raise Exception(message)
                    aliases[entry] = developer_script_class

                else:
                    entry = (instance.alias,)
                    if entry in scripting_groups:
                        message = 'alias conflict between {}'
                        message += ' and scripting group {!r}'
                        message = message.format(
                            developer_script_class.__name__,
                            instance.alias,
                            )
                        raise Exception(message)
                    if entry in aliases:
                        message = 'alias conflict be {} and {}'
                        message = message.format(
                            developer_script_class.__name__,
                            aliases[entry],
                            )
                        raise Exception(message)
                    aliases[(instance.alias,)] = developer_script_class

            else:
                if instance.program_name in scripting_groups:
                    message = 'Alias conflict between {}'
                    message += ' and scripting group {!r}'
                    message = message.format(
                        developer_script_class.__name__,
                        instance.program_name,
                        )
                    raise Exception(message)
                aliases[(instance.program_name,)] = developer_script_class

        aliasdict = {}

        for key, value in aliases.items():
            if len(key) == 1:
                aliasdict[key[0]] = value
            else:
                if key[0] not in aliasdict:
                    aliasdict[key[0]] = { }
                aliasdict[key[0]][key[1]] = value

        return aliasdict

    @property
    def developer_script_classes(self):
        r'''Developer scripts classes.
        '''
        from abjad.tools import developerscripttools
        classes = developerscripttools.get_developer_script_classes()
        classes.remove(type(self))
        return classes

    @property
    def developer_script_program_names(self):
        r'''Developer script program names.
        '''
        program_names = {}
        for developer_script_class in self.developer_script_classes:
            instance = developer_script_class()
            program_names[instance.program_name] = developer_script_class
        return program_names

    @property
    def long_description(self):
        r'''Long description.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description.
        '''
        return 'Entry-point to Abjad developer scripts catalog.'

    @property
    def version(self):
        r'''Version.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.
        '''
        args, unknown_args = args

        if args.subparser_name == 'help':
            aliases = self.developer_script_aliases
            program_names = self.developer_script_program_names
            developer_script_class = None
            if len(unknown_args) == 2 and \
                unknown_args[0] in aliases and \
                unknown_args[1] in aliases[unknown_args[0]]:
                developer_script_class = \
                    aliases[unknown_args[0]][unknown_args[1]]
            elif len(unknown_args) == 1 and \
                unknown_args[0] in aliases and \
                not isinstance(aliases[unknown_args[0]], dict):
                developer_script_class = aliases[unknown_args[0]]
            elif len(unknown_args) == 1 and \
                unknown_args[0] in program_names:
                developer_script_class = program_names[unknown_args[0]]

            if developer_script_class:
                instance = developer_script_class()
                print(instance.formatted_help)
            else:
                print('Cannot resolve {} to subcommand.'.format(unknown_args))

        elif args.subparser_name == 'list':
            entries = []
            for developer_script_class in self.developer_script_classes:
                instance = developer_script_class()
                alias = ''
                if instance.alias is not None:
                    if instance.scripting_group is not None:
                        alias = '\n[{} {}]'.format(
                            instance.scripting_group, instance.alias)
                    else:
                        alias = '\n[{}]'.format(instance.alias)
                entries.append('{}{}\n\t{}'.format(
                    instance.program_name, alias, instance.short_description))
            print('')
            print('\n\n'.join(entries))
            print('')

        else:
            if hasattr(args, 'subsubparser_name'):
                developer_script_class = self.developer_script_aliases[
                    args.subparser_name][args.subsubparser_name]
            else:
                developer_script_class = \
                    self.developer_script_aliases[args.subparser_name]
            instance = developer_script_class()
            instance(unknown_args)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.
        '''

        subparsers = parser.add_subparsers(
            dest='subparser_name',
            title='subcommands',
            )

        info_subparser = subparsers.add_parser('help',
            add_help=False,
            help='print subcommand help'
            )

        list_subparser = subparsers.add_parser('list',
            add_help=False,
            help='list subcommands',
            )

        aliasdict = self.developer_script_aliases
        for key in sorted(aliasdict):
            if not isinstance(aliasdict[key], dict):
                developer_script_class = aliasdict[key]
                instance = developer_script_class()
                class_subparser = subparsers.add_parser(key,
                    add_help=False,
                    help=instance.short_description,
                    )
            else:
                group_subparser = subparsers.add_parser(key,
                    help='"{}"-related subcommands'.format(key),
                    )
                group_subparsers = group_subparser.add_subparsers(
                    dest='subsubparser_name'.format(key),
                    title='{} subcommands'.format(key),
                    )
                for subkey in sorted(aliasdict[key]):
                    developer_script_class = aliasdict[key][subkey]
                    instance = developer_script_class()
                    group_subparsers.add_parser(subkey,
                        add_help=False,
                        help=instance.short_description
                        )


# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_named_interval_classes(expr, markup_direction=Up):
    r"""Label leaves in `expr` with named interval classes:

    ::

        >>> notes = scoretools.make_notes([0, 25, 11, -4, -14, -13, 9, 10, 6, 5], [Duration(1, 8)])
        >>> staff = Staff(notes)
        >>> labeltools.label_leaves_in_expr_with_named_interval_classes(staff)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'8 ^ \markup { +aug8 }
            cs'''8 ^ \markup { -M2 }
            b'8 ^ \markup { -aug2 }
            af8 ^ \markup { -m7 }
            bf,8 ^ \markup { aug1 }
            b,8 ^ \markup { +m7 }
            a'8 ^ \markup { +m2 }
            bf'8 ^ \markup { -dim4 }
            fs'8 ^ \markup { aug1 }
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    """

    for note in iterate(expr).by_class(scoretools.Note):
        logical_voice_iterator = iterate(note).by_logical_voice_from_component(
            scoretools.Leaf,
            )
        try:
            next(logical_voice_iterator)
            next_leaf = next(logical_voice_iterator)
            if isinstance(next_leaf, scoretools.Note):
                mdi = pitchtools.NamedInterval.from_pitch_carriers(
                    note, next_leaf)
                mdic = pitchtools.NamedIntervalClass(mdi)
                markup = markuptools.Markup(mdic, markup_direction)
                attach(markup, note)
        except StopIteration:
            pass



# -*- encoding: utf-8 -*-


def is_assignable_integer(expr):
    r'''Is true when `expr` is equivalent to an integer and
    can be written without recourse to ties.

    ::

        >>> for n in range(0, 16 + 1):
        ...     print('%s\t%s' % (n, mathtools.is_assignable_integer(n)))
        ... 
        0  False
        1  True
        2  True
        3  True
        4  True
        5  False
        6  True
        7  True
        8  True
        9  False
        10 False
        11 False
        12 True
        13 False
        14 True
        15 True
        16 True

    Otherwise false.

    Returns boolean.
    '''
    from abjad.tools import mathtools

    if isinstance(expr, int):
        if 0 < expr:
            if not '01' in mathtools.integer_to_binary_string(expr):
                return True
    return False
